# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2025 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "ServerSSLCert.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.cert


# interface
class ServerSSLCert(Interface):
    idlType = "cert.ServerSSLCert:4.0.0"

    SUCCESS = 0

    ERR_BUSY = 1

    ERR_CSR_OR_CERT_PENDING = 101

    ERR_KEY_MISSING = 200

    ERR_CERT_MISSING = 201

    ERR_CERT_FORMAT_INVALID = 202

    ERR_CERT_KEY_MISMATCH = 203

    ERR_KEY_FORMAT_INVALID = 204

    ERR_GEN_KEY_LEN_INVALID = 100

    ERR_GEN_CSR_OR_CERT_PENDING = 101

    ERR_GEN_KEY_GEN_FAILED = 102

    ERR_GEN_KEY_TYPE_INVALID = 103

    ERR_GEN_ELLIPTIC_CURVE_INVALID = 104

    ERR_GEN_VALIDITY_OUT_OF_RANGE = 105

    ERR_INSTALL_KEY_MISSING = 200

    ERR_INSTALL_CERT_MISSING = 201

    ERR_INSTALL_CERT_FORMAT_INVALID = 202

    ERR_INSTALL_CERT_KEY_MISMATCH = 203

    ERR_INSTALL_KEY_FORMAT_INVALID = 204

    # structure
    class CommonAttributes(Structure):
        idlType = "cert.ServerSSLCert_4_0_0.CommonAttributes:1.0.0"
        elements = ["country", "stateOrProvince", "locality", "organization", "organizationalUnit", "commonName", "emailAddress"]

        def __init__(self, country="", stateOrProvince="", locality="", organization="", organizationalUnit="", commonName="", emailAddress=""):
            if not typecheck._is_int(country):
                typecheck.is_string(country, AssertionError)
            if not typecheck._is_int(stateOrProvince):
                typecheck.is_string(stateOrProvince, AssertionError)
            if not typecheck._is_int(locality):
                typecheck.is_string(locality, AssertionError)
            if not typecheck._is_int(organization):
                typecheck.is_string(organization, AssertionError)
            if not typecheck._is_int(organizationalUnit):
                typecheck.is_string(organizationalUnit, AssertionError)
            if not typecheck._is_int(commonName):
                typecheck.is_string(commonName, AssertionError)
            if not typecheck._is_int(emailAddress):
                typecheck.is_string(emailAddress, AssertionError)

            self.country = country
            self.stateOrProvince = stateOrProvince
            self.locality = locality
            self.organization = organization
            self.organizationalUnit = organizationalUnit
            self.commonName = commonName
            self.emailAddress = emailAddress

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                country = json['country'] if 'country' in json or not useDefaults else "",
                stateOrProvince = json['stateOrProvince'] if 'stateOrProvince' in json or not useDefaults else "",
                locality = json['locality'] if 'locality' in json or not useDefaults else "",
                organization = json['organization'] if 'organization' in json or not useDefaults else "",
                organizationalUnit = json['organizationalUnit'] if 'organizationalUnit' in json or not useDefaults else "",
                commonName = json['commonName'] if 'commonName' in json or not useDefaults else "",
                emailAddress = json['emailAddress'] if 'emailAddress' in json or not useDefaults else "",
            )
            return obj

        def encode(self):
            json = {}
            json['country'] = str(self.country)
            json['stateOrProvince'] = str(self.stateOrProvince)
            json['locality'] = str(self.locality)
            json['organization'] = str(self.organization)
            json['organizationalUnit'] = str(self.organizationalUnit)
            json['commonName'] = str(self.commonName)
            json['emailAddress'] = str(self.emailAddress)
            return json

    # enumeration
    class KeyType(Enumeration):
        idlType = "cert.ServerSSLCert_4_0_0.KeyType:1.0.0"
        values = ["KEY_TYPE_UNKNOWN", "KEY_TYPE_RSA", "KEY_TYPE_ECDSA"]

    KeyType.KEY_TYPE_UNKNOWN = KeyType(0)
    KeyType.KEY_TYPE_RSA = KeyType(1)
    KeyType.KEY_TYPE_ECDSA = KeyType(2)

    # enumeration
    class EllipticCurve(Enumeration):
        idlType = "cert.ServerSSLCert_4_0_0.EllipticCurve:1.0.0"
        values = ["EC_CURVE_UNKNOWN", "EC_CURVE_NIST_P256", "EC_CURVE_NIST_P384", "EC_CURVE_NIST_P521"]

    EllipticCurve.EC_CURVE_UNKNOWN = EllipticCurve(0)
    EllipticCurve.EC_CURVE_NIST_P256 = EllipticCurve(1)
    EllipticCurve.EC_CURVE_NIST_P384 = EllipticCurve(2)
    EllipticCurve.EC_CURVE_NIST_P521 = EllipticCurve(3)

    # structure
    class KeyInfo(Structure):
        idlType = "cert.ServerSSLCert_4_0_0.KeyInfo:1.0.0"
        elements = ["type", "ecCurve", "rsaKeyLength", "inSecureElement"]

        def __init__(self, type=None, ecCurve=None, rsaKeyLength=0, inSecureElement=False):
            if type is None:
                type = raritan.rpc.cert.ServerSSLCert.KeyType.KEY_TYPE_UNKNOWN
            if ecCurve is None:
                ecCurve = raritan.rpc.cert.ServerSSLCert.EllipticCurve.EC_CURVE_UNKNOWN
            typecheck.is_enum(type, raritan.rpc.cert.ServerSSLCert.KeyType, AssertionError)
            typecheck.is_enum(ecCurve, raritan.rpc.cert.ServerSSLCert.EllipticCurve, AssertionError)
            typecheck.is_int(rsaKeyLength, AssertionError)
            typecheck.is_bool(inSecureElement, AssertionError)

            self.type = type
            self.ecCurve = ecCurve
            self.rsaKeyLength = rsaKeyLength
            self.inSecureElement = inSecureElement

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                type = raritan.rpc.cert.ServerSSLCert.KeyType.decode(json['type']) if 'type' in json or not useDefaults else raritan.rpc.cert.ServerSSLCert.KeyType.KEY_TYPE_UNKNOWN,
                ecCurve = raritan.rpc.cert.ServerSSLCert.EllipticCurve.decode(json['ecCurve']) if 'ecCurve' in json or not useDefaults else raritan.rpc.cert.ServerSSLCert.EllipticCurve.EC_CURVE_UNKNOWN,
                rsaKeyLength = json['rsaKeyLength'] if 'rsaKeyLength' in json or not useDefaults else 0,
                inSecureElement = json['inSecureElement'] if 'inSecureElement' in json or not useDefaults else False,
            )
            return obj

        def encode(self):
            json = {}
            json['type'] = raritan.rpc.cert.ServerSSLCert.KeyType.encode(self.type)
            json['ecCurve'] = raritan.rpc.cert.ServerSSLCert.EllipticCurve.encode(self.ecCurve)
            json['rsaKeyLength'] = self.rsaKeyLength
            json['inSecureElement'] = self.inSecureElement
            return json

    # structure
    class ReqInfo(Structure):
        idlType = "cert.ServerSSLCert_4_0_0.ReqInfo:1.0.0"
        elements = ["subject", "names", "keyInfo"]

        def __init__(self, subject=None, names=[], keyInfo=None):
            if subject is None:
                subject = raritan.rpc.cert.ServerSSLCert.CommonAttributes()
            if keyInfo is None:
                keyInfo = raritan.rpc.cert.ServerSSLCert.KeyInfo()
            typecheck.is_struct(subject, raritan.rpc.cert.ServerSSLCert.CommonAttributes, AssertionError)
            for x0 in names:
                if not typecheck._is_int(x0):
                    typecheck.is_string(x0, AssertionError)
            typecheck.is_struct(keyInfo, raritan.rpc.cert.ServerSSLCert.KeyInfo, AssertionError)

            self.subject = subject
            self.names = names
            self.keyInfo = keyInfo

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                subject = raritan.rpc.cert.ServerSSLCert.CommonAttributes.decode(json['subject'], agent, useDefaults=useDefaults) if 'subject' in json or not useDefaults else raritan.rpc.cert.ServerSSLCert.CommonAttributes(),
                names = [x0 for x0 in json['names']] if 'names' in json or not useDefaults else [],
                keyInfo = raritan.rpc.cert.ServerSSLCert.KeyInfo.decode(json['keyInfo'], agent, useDefaults=useDefaults) if 'keyInfo' in json or not useDefaults else raritan.rpc.cert.ServerSSLCert.KeyInfo(),
            )
            return obj

        def encode(self):
            json = {}
            json['subject'] = raritan.rpc.cert.ServerSSLCert.CommonAttributes.encode(self.subject)
            json['names'] = [str(x0) for x0 in self.names]
            json['keyInfo'] = raritan.rpc.cert.ServerSSLCert.KeyInfo.encode(self.keyInfo)
            return json

    # structure
    class CertInfo(Structure):
        idlType = "cert.ServerSSLCert_4_0_0.CertInfo:1.0.0"
        elements = ["subject", "issuer", "names", "invalidBefore", "invalidAfter", "serialNumber", "keyInfo"]

        def __init__(self, subject=None, issuer=None, names=[], invalidBefore="", invalidAfter="", serialNumber="", keyInfo=None):
            if subject is None:
                subject = raritan.rpc.cert.ServerSSLCert.CommonAttributes()
            if issuer is None:
                issuer = raritan.rpc.cert.ServerSSLCert.CommonAttributes()
            if keyInfo is None:
                keyInfo = raritan.rpc.cert.ServerSSLCert.KeyInfo()
            typecheck.is_struct(subject, raritan.rpc.cert.ServerSSLCert.CommonAttributes, AssertionError)
            typecheck.is_struct(issuer, raritan.rpc.cert.ServerSSLCert.CommonAttributes, AssertionError)
            for x0 in names:
                if not typecheck._is_int(x0):
                    typecheck.is_string(x0, AssertionError)
            if not typecheck._is_int(invalidBefore):
                typecheck.is_string(invalidBefore, AssertionError)
            if not typecheck._is_int(invalidAfter):
                typecheck.is_string(invalidAfter, AssertionError)
            if not typecheck._is_int(serialNumber):
                typecheck.is_string(serialNumber, AssertionError)
            typecheck.is_struct(keyInfo, raritan.rpc.cert.ServerSSLCert.KeyInfo, AssertionError)

            self.subject = subject
            self.issuer = issuer
            self.names = names
            self.invalidBefore = invalidBefore
            self.invalidAfter = invalidAfter
            self.serialNumber = serialNumber
            self.keyInfo = keyInfo

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                subject = raritan.rpc.cert.ServerSSLCert.CommonAttributes.decode(json['subject'], agent, useDefaults=useDefaults) if 'subject' in json or not useDefaults else raritan.rpc.cert.ServerSSLCert.CommonAttributes(),
                issuer = raritan.rpc.cert.ServerSSLCert.CommonAttributes.decode(json['issuer'], agent, useDefaults=useDefaults) if 'issuer' in json or not useDefaults else raritan.rpc.cert.ServerSSLCert.CommonAttributes(),
                names = [x0 for x0 in json['names']] if 'names' in json or not useDefaults else [],
                invalidBefore = json['invalidBefore'] if 'invalidBefore' in json or not useDefaults else "",
                invalidAfter = json['invalidAfter'] if 'invalidAfter' in json or not useDefaults else "",
                serialNumber = json['serialNumber'] if 'serialNumber' in json or not useDefaults else "",
                keyInfo = raritan.rpc.cert.ServerSSLCert.KeyInfo.decode(json['keyInfo'], agent, useDefaults=useDefaults) if 'keyInfo' in json or not useDefaults else raritan.rpc.cert.ServerSSLCert.KeyInfo(),
            )
            return obj

        def encode(self):
            json = {}
            json['subject'] = raritan.rpc.cert.ServerSSLCert.CommonAttributes.encode(self.subject)
            json['issuer'] = raritan.rpc.cert.ServerSSLCert.CommonAttributes.encode(self.issuer)
            json['names'] = [str(x0) for x0 in self.names]
            json['invalidBefore'] = str(self.invalidBefore)
            json['invalidAfter'] = str(self.invalidAfter)
            json['serialNumber'] = str(self.serialNumber)
            json['keyInfo'] = raritan.rpc.cert.ServerSSLCert.KeyInfo.encode(self.keyInfo)
            return json

    # structure
    class Info(Structure):
        idlType = "cert.ServerSSLCert_4_0_0.Info:1.0.0"
        elements = ["havePendingReq", "havePendingCert", "pendingReqInfo", "pendingCertInfo", "pendingCertChainInfos", "activeCertInfo", "activeCertChainInfos", "maxSignDays"]

        def __init__(self, havePendingReq=False, havePendingCert=False, pendingReqInfo=None, pendingCertInfo=None, pendingCertChainInfos=[], activeCertInfo=None, activeCertChainInfos=[], maxSignDays=0):
            if pendingReqInfo is None:
                pendingReqInfo = raritan.rpc.cert.ServerSSLCert.ReqInfo()
            if pendingCertInfo is None:
                pendingCertInfo = raritan.rpc.cert.ServerSSLCert.CertInfo()
            if activeCertInfo is None:
                activeCertInfo = raritan.rpc.cert.ServerSSLCert.CertInfo()
            typecheck.is_bool(havePendingReq, AssertionError)
            typecheck.is_bool(havePendingCert, AssertionError)
            typecheck.is_struct(pendingReqInfo, raritan.rpc.cert.ServerSSLCert.ReqInfo, AssertionError)
            typecheck.is_struct(pendingCertInfo, raritan.rpc.cert.ServerSSLCert.CertInfo, AssertionError)
            for x0 in pendingCertChainInfos:
                typecheck.is_struct(x0, raritan.rpc.cert.ServerSSLCert.CertInfo, AssertionError)
            typecheck.is_struct(activeCertInfo, raritan.rpc.cert.ServerSSLCert.CertInfo, AssertionError)
            for x0 in activeCertChainInfos:
                typecheck.is_struct(x0, raritan.rpc.cert.ServerSSLCert.CertInfo, AssertionError)
            typecheck.is_int(maxSignDays, AssertionError)

            self.havePendingReq = havePendingReq
            self.havePendingCert = havePendingCert
            self.pendingReqInfo = pendingReqInfo
            self.pendingCertInfo = pendingCertInfo
            self.pendingCertChainInfos = pendingCertChainInfos
            self.activeCertInfo = activeCertInfo
            self.activeCertChainInfos = activeCertChainInfos
            self.maxSignDays = maxSignDays

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                havePendingReq = json['havePendingReq'] if 'havePendingReq' in json or not useDefaults else False,
                havePendingCert = json['havePendingCert'] if 'havePendingCert' in json or not useDefaults else False,
                pendingReqInfo = raritan.rpc.cert.ServerSSLCert.ReqInfo.decode(json['pendingReqInfo'], agent, useDefaults=useDefaults) if 'pendingReqInfo' in json or not useDefaults else raritan.rpc.cert.ServerSSLCert.ReqInfo(),
                pendingCertInfo = raritan.rpc.cert.ServerSSLCert.CertInfo.decode(json['pendingCertInfo'], agent, useDefaults=useDefaults) if 'pendingCertInfo' in json or not useDefaults else raritan.rpc.cert.ServerSSLCert.CertInfo(),
                pendingCertChainInfos = [raritan.rpc.cert.ServerSSLCert.CertInfo.decode(x0, agent, useDefaults=useDefaults) for x0 in json['pendingCertChainInfos']] if 'pendingCertChainInfos' in json or not useDefaults else [],
                activeCertInfo = raritan.rpc.cert.ServerSSLCert.CertInfo.decode(json['activeCertInfo'], agent, useDefaults=useDefaults) if 'activeCertInfo' in json or not useDefaults else raritan.rpc.cert.ServerSSLCert.CertInfo(),
                activeCertChainInfos = [raritan.rpc.cert.ServerSSLCert.CertInfo.decode(x0, agent, useDefaults=useDefaults) for x0 in json['activeCertChainInfos']] if 'activeCertChainInfos' in json or not useDefaults else [],
                maxSignDays = json['maxSignDays'] if 'maxSignDays' in json or not useDefaults else 0,
            )
            return obj

        def encode(self):
            json = {}
            json['havePendingReq'] = self.havePendingReq
            json['havePendingCert'] = self.havePendingCert
            json['pendingReqInfo'] = raritan.rpc.cert.ServerSSLCert.ReqInfo.encode(self.pendingReqInfo)
            json['pendingCertInfo'] = raritan.rpc.cert.ServerSSLCert.CertInfo.encode(self.pendingCertInfo)
            json['pendingCertChainInfos'] = [raritan.rpc.cert.ServerSSLCert.CertInfo.encode(x0) for x0 in self.pendingCertChainInfos]
            json['activeCertInfo'] = raritan.rpc.cert.ServerSSLCert.CertInfo.encode(self.activeCertInfo)
            json['activeCertChainInfos'] = [raritan.rpc.cert.ServerSSLCert.CertInfo.encode(x0) for x0 in self.activeCertChainInfos]
            json['maxSignDays'] = self.maxSignDays
            return json

    class _getSupportedKeyInfos(Interface.Method):
        name = 'getSupportedKeyInfos'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = [raritan.rpc.cert.ServerSSLCert.KeyInfo.decode(x0, agent, useDefaults=useDefaults) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.cert.ServerSSLCert.KeyInfo, DecodeException)
            return _ret_

    class _generateUnsignedKeyPair(Interface.Method):
        name = 'generateUnsignedKeyPair'

        @staticmethod
        def encode(reqInfo, challenge):
            typecheck.is_struct(reqInfo, raritan.rpc.cert.ServerSSLCert.ReqInfo, AssertionError)
            if not typecheck._is_int(challenge):
                typecheck.is_string(challenge, AssertionError)
            args = {}
            args['reqInfo'] = raritan.rpc.cert.ServerSSLCert.ReqInfo.encode(reqInfo)
            args['challenge'] = str(challenge)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _generateSelfSignedKeyPair(Interface.Method):
        name = 'generateSelfSignedKeyPair'

        @staticmethod
        def encode(reqInfo, days):
            typecheck.is_struct(reqInfo, raritan.rpc.cert.ServerSSLCert.ReqInfo, AssertionError)
            typecheck.is_int(days, AssertionError)
            args = {}
            args['reqInfo'] = raritan.rpc.cert.ServerSSLCert.ReqInfo.encode(reqInfo)
            args['days'] = days
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _deletePending(Interface.Method):
        name = 'deletePending'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            return None

    class _getInfo(Interface.Method):
        name = 'getInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            info = raritan.rpc.cert.ServerSSLCert.Info.decode(rsp['info'], agent, useDefaults=useDefaults)
            typecheck.is_struct(info, raritan.rpc.cert.ServerSSLCert.Info, DecodeException)
            return info

    class _getActiveCertChainPEM(Interface.Method):
        name = 'getActiveCertChainPEM'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            if not typecheck._is_int(_ret_):
                typecheck.is_string(_ret_, DecodeException)
            return _ret_

    class _getActiveKeyPEM(Interface.Method):
        name = 'getActiveKeyPEM'

        @staticmethod
        def encode(keyPassword):
            if not typecheck._is_int(keyPassword):
                typecheck.is_string(keyPassword, AssertionError)
            args = {}
            args['keyPassword'] = str(keyPassword)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            if not typecheck._is_int(_ret_):
                typecheck.is_string(_ret_, DecodeException)
            return _ret_

    class _getPendingRequestPEM(Interface.Method):
        name = 'getPendingRequestPEM'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            if not typecheck._is_int(_ret_):
                typecheck.is_string(_ret_, DecodeException)
            return _ret_

    class _getPendingCertChainPEM(Interface.Method):
        name = 'getPendingCertChainPEM'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            if not typecheck._is_int(_ret_):
                typecheck.is_string(_ret_, DecodeException)
            return _ret_

    class _getPendingKeyPEM(Interface.Method):
        name = 'getPendingKeyPEM'

        @staticmethod
        def encode(keyPassword):
            if not typecheck._is_int(keyPassword):
                typecheck.is_string(keyPassword, AssertionError)
            args = {}
            args['keyPassword'] = str(keyPassword)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            if not typecheck._is_int(_ret_):
                typecheck.is_string(_ret_, DecodeException)
            return _ret_

    class _setPendingCertChainPEM(Interface.Method):
        name = 'setPendingCertChainPEM'

        @staticmethod
        def encode(certChain):
            if not typecheck._is_int(certChain):
                typecheck.is_string(certChain, AssertionError)
            args = {}
            args['certChain'] = str(certChain)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _setPendingKeyAndCertChainPEM(Interface.Method):
        name = 'setPendingKeyAndCertChainPEM'

        @staticmethod
        def encode(key, certChain, keyPassword):
            if not typecheck._is_int(key):
                typecheck.is_string(key, AssertionError)
            if not typecheck._is_int(certChain):
                typecheck.is_string(certChain, AssertionError)
            if not typecheck._is_int(keyPassword):
                typecheck.is_string(keyPassword, AssertionError)
            args = {}
            args['key'] = str(key)
            args['certChain'] = str(certChain)
            args['keyPassword'] = str(keyPassword)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _installPendingKeyPair(Interface.Method):
        name = 'installPendingKeyPair'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(ServerSSLCert, self).__init__(target, agent)
        self.getSupportedKeyInfos = ServerSSLCert._getSupportedKeyInfos(self)
        self.generateUnsignedKeyPair = ServerSSLCert._generateUnsignedKeyPair(self)
        self.generateSelfSignedKeyPair = ServerSSLCert._generateSelfSignedKeyPair(self)
        self.deletePending = ServerSSLCert._deletePending(self)
        self.getInfo = ServerSSLCert._getInfo(self)
        self.getActiveCertChainPEM = ServerSSLCert._getActiveCertChainPEM(self)
        self.getActiveKeyPEM = ServerSSLCert._getActiveKeyPEM(self)
        self.getPendingRequestPEM = ServerSSLCert._getPendingRequestPEM(self)
        self.getPendingCertChainPEM = ServerSSLCert._getPendingCertChainPEM(self)
        self.getPendingKeyPEM = ServerSSLCert._getPendingKeyPEM(self)
        self.setPendingCertChainPEM = ServerSSLCert._setPendingCertChainPEM(self)
        self.setPendingKeyAndCertChainPEM = ServerSSLCert._setPendingKeyAndCertChainPEM(self)
        self.installPendingKeyPair = ServerSSLCert._installPendingKeyPair(self)

# from raritan/rpc/cert/__extend__.py
def upload(agent, certData, keyData=None):
    """
    Method to upload certificates

    - **parameters**, **return**

    :param agent: An agent instance for the device where the certificate should be uploaded
    :param certData: The binary data of the certificate
    :param keyData: The binary data of the key (optional, default: None)
    :return: the response code

    - **Example**
        :Example:

        from raritan import rpc
        from raritan.rpc import cert

        agent = rpc.Agent("https", "my-pdu.example.com", "admin", "raritan")
        ssl_proxy = cert.ServerSSLCert("/server_ssl_cert", agent)

        # read files in binary mode
        certFile = open("my-cert.crt", "rb")
        keyFile = open("my-key.key", "rb")
        # upload
        code = cert.upload(agent, certFile.read(), keyFile.read())
        # view code
        print(code)
    """
    target = "cgi-bin/server_ssl_cert_upload.cgi"
    formdata = [dict(data=certData, filename="cert-file.crt", formname="cert_file", mimetype="application/octet-stream")]
    if keyData:
        formdata.append(dict(data=keyData, filename="key-file.key", formname="key_file", mimetype="application/octet-stream"))
    response = agent.form_data_file(target, formdata)
    # extract the result from "return_code=<code>" from body
    respBody = response["body"].split("=")
    return int(respBody[1])

def download(agent, tag="active_cert"):
    """
    Method to download the server cert

    **parameters**

    :param agent: An agent instance from the device where the certificate file should be downloaded
    :param tag: The tag to define what should be downloaded
    (default: "active_cert", values: "active_cert","active_key","new_cert","new_key","new_req")
    :return: returns the certificate data

    **Example**
        :Example:

        from raritan import rpc
        from raritan.rpc import cert

        agent = rpc.Agent("https", "my-pdu.example.com", "admin", "raritan")
        # download
        cert = cert.download(agent, "active_cert")
        print(cert)
    """
    target = "cgi-bin/server_ssl_cert_download.cgi?tag=%s" % tag
    return agent.get(target)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2025 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "TestDisplay.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.test


# interface
class Display(Interface):
    idlType = "test.Display:2.0.0"

    # enumeration
    class Orientation(Enumeration):
        idlType = "test.Display_2_0_0.Orientation:1.0.0"
        values = ["NORMAL", "FLIPPED", "LEFT", "RIGHT"]

    Orientation.NORMAL = Orientation(0)
    Orientation.FLIPPED = Orientation(1)
    Orientation.LEFT = Orientation(2)
    Orientation.RIGHT = Orientation(3)

    # structure
    class Info(Structure):
        idlType = "test.Display_2_0_0.Info:1.0.0"
        elements = ["type", "address", "options", "orientation"]

        def __init__(self, type="", address="", options={}, orientation=None):
            if orientation is None:
                orientation = raritan.rpc.test.Display.Orientation.NORMAL
            if not typecheck._is_int(type):
                typecheck.is_string(type, AssertionError)
            if not typecheck._is_int(address):
                typecheck.is_string(address, AssertionError)
            typecheck.is_enum(orientation, raritan.rpc.test.Display.Orientation, AssertionError)

            self.type = type
            self.address = address
            self.options = options
            self.orientation = orientation

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                type = json['type'] if 'type' in json or not useDefaults else "",
                address = json['address'] if 'address' in json or not useDefaults else "",
                options = dict([(
                    elem['key'],
                    elem['value'])
                    for elem in json['options']]) if 'options' in json or not useDefaults else {},
                orientation = raritan.rpc.test.Display.Orientation.decode(json['orientation']) if 'orientation' in json or not useDefaults else raritan.rpc.test.Display.Orientation.NORMAL,
            )
            return obj

        def encode(self):
            json = {}
            json['type'] = str(self.type)
            json['address'] = str(self.address)
            json['options'] = [dict(
                key = str(k),
                value = str(v))
                for k, v in self.options.items()]
            json['orientation'] = raritan.rpc.test.Display.Orientation.encode(self.orientation)
            return json

    class _getInfo(Interface.Method):
        name = 'getInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.test.Display.Info.decode(rsp['_ret_'], agent, useDefaults=useDefaults)
            typecheck.is_struct(_ret_, raritan.rpc.test.Display.Info, DecodeException)
            return _ret_

    class _testSequence(Interface.Method):
        name = 'testSequence'

        @staticmethod
        def encode(cycleTime_ms):
            typecheck.is_int(cycleTime_ms, AssertionError)
            args = {}
            args['cycleTime_ms'] = cycleTime_ms
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            return None

    # enumeration
    class TestStatus(Enumeration):
        idlType = "test.Display_2_0_0.TestStatus:1.0.0"
        values = ["TEST_IDLE", "TEST_BUSY", "TEST_PASSED", "TEST_FAILED"]

    TestStatus.TEST_IDLE = TestStatus(0)
    TestStatus.TEST_BUSY = TestStatus(1)
    TestStatus.TEST_PASSED = TestStatus(2)
    TestStatus.TEST_FAILED = TestStatus(3)

    class _enterTestMode(Interface.Method):
        name = 'enterTestMode'

        @staticmethod
        def encode(showColorNames):
            typecheck.is_bool(showColorNames, AssertionError)
            args = {}
            args['showColorNames'] = showColorNames
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            return None

    class _getTestStatus(Interface.Method):
        name = 'getTestStatus'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.test.Display.TestStatus.decode(rsp['_ret_'])
            typecheck.is_enum(_ret_, raritan.rpc.test.Display.TestStatus, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(Display, self).__init__(target, agent)
        self.getInfo = Display._getInfo(self)
        self.testSequence = Display._testSequence(self)
        self.enterTestMode = Display._enterTestMode(self)
        self.getTestStatus = Display._getTestStatus(self)

#
# Section generated by IdlC from "TestUnit.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.test


# interface
class Unit(Interface):
    idlType = "test.Unit:2.0.0"

    class _getDisplays(Interface.Method):
        name = 'getDisplays'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = [Interface.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_interface(x0, raritan.rpc.test.Display, DecodeException)
            return _ret_

    class _getButtonStates(Interface.Method):
        name = 'getButtonStates'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = [x0 for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_bool(x0, DecodeException)
            return _ret_

    class _setBuzzer(Interface.Method):
        name = 'setBuzzer'

        @staticmethod
        def encode(isOn):
            typecheck.is_bool(isOn, AssertionError)
            args = {}
            args['isOn'] = isOn
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            return None

    class _resetAllSubControllers(Interface.Method):
        name = 'resetAllSubControllers'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            return None

    class _triggerSubControllerWatchdog(Interface.Method):
        name = 'triggerSubControllerWatchdog'

        @staticmethod
        def encode(rs485Addr):
            typecheck.is_int(rs485Addr, AssertionError)
            args = {}
            args['rs485Addr'] = rs485Addr
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            return None
    def __init__(self, target, agent):
        super(Unit, self).__init__(target, agent)
        self.getDisplays = Unit._getDisplays(self)
        self.getButtonStates = Unit._getButtonStates(self)
        self.setBuzzer = Unit._setBuzzer(self)
        self.resetAllSubControllers = Unit._resetAllSubControllers(self)
        self.triggerSubControllerWatchdog = Unit._triggerSubControllerWatchdog(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2025 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "CascadeManager.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.cascading

import raritan.rpc.event

import raritan.rpc.idl


# interface
class CascadeManager(Interface):
    idlType = "cascading.CascadeManager:2.0.2"

    NO_ERROR = 0

    ERR_INVALID_PARAM = 1

    ERR_UNSUPPORTED_ON_PRIMARY_UNIT = 2

    ERR_UNSUPPORTED_ON_LINK_UNIT = 3

    ERR_LINK_ID_IN_USE = 4

    ERR_HOST_IN_USE = 5

    ERR_LINK_UNIT_UNREACHABLE = 6

    ERR_LINK_UNIT_ACCESS_DENIED = 7

    ERR_LINK_UNIT_REFUSED = 8

    ERR_UNIT_BUSY = 9

    ERR_NOT_SUPPORTED = 10

    ERR_PASSWORD_CHANGE_REQUIRED = 11

    ERR_PASSWORD_POLICY = 12

    ERR_LINK_UNIT_COMM_FAILED = 13

    ERR_LINK_UNIT_NOT_SUPPORTED = 14

    ERR_FIRMWARE_VERSION_MISMATCH = 15

    # structure
    class PrimaryUnitSettings(Structure):
        idlType = "cascading.CascadeManager_2_0_2.PrimaryUnitSettings:1.0.0"
        elements = ["caCertChain", "allowOffTimeRangeCerts"]

        def __init__(self, caCertChain="", allowOffTimeRangeCerts=False):
            if not typecheck._is_int(caCertChain):
                typecheck.is_string(caCertChain, AssertionError)
            typecheck.is_bool(allowOffTimeRangeCerts, AssertionError)

            self.caCertChain = caCertChain
            self.allowOffTimeRangeCerts = allowOffTimeRangeCerts

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                caCertChain = json['caCertChain'] if 'caCertChain' in json or not useDefaults else "",
                allowOffTimeRangeCerts = json['allowOffTimeRangeCerts'] if 'allowOffTimeRangeCerts' in json or not useDefaults else False,
            )
            return obj

        def encode(self):
            json = {}
            json['caCertChain'] = str(self.caCertChain)
            json['allowOffTimeRangeCerts'] = self.allowOffTimeRangeCerts
            return json

    # enumeration
    class Role(Enumeration):
        idlType = "cascading.CascadeManager_2_0_2.Role:1.0.0"
        values = ["STANDALONE", "PRIMARY_UNIT", "LINK_UNIT"]

    Role.STANDALONE = Role(0)
    Role.PRIMARY_UNIT = Role(1)
    Role.LINK_UNIT = Role(2)

    # enumeration
    class LinkUnitType(Enumeration):
        idlType = "cascading.CascadeManager_2_0_2.LinkUnitType:1.0.0"
        values = ["NETWORK", "SECURE_SERIAL"]

    LinkUnitType.NETWORK = LinkUnitType(0)
    LinkUnitType.SECURE_SERIAL = LinkUnitType(1)

    # enumeration
    class LinkUnitStatus(Enumeration):
        idlType = "cascading.CascadeManager_2_0_2.LinkUnitStatus:1.0.0"
        values = ["UNKNOWN", "OK", "UNREACHABLE", "ACCESS_DENIED", "FIRMWARE_UPDATE", "FIRMWARE_MISMATCH", "PENDING"]

    LinkUnitStatus.UNKNOWN = LinkUnitStatus(0)
    LinkUnitStatus.OK = LinkUnitStatus(1)
    LinkUnitStatus.UNREACHABLE = LinkUnitStatus(2)
    LinkUnitStatus.ACCESS_DENIED = LinkUnitStatus(3)
    LinkUnitStatus.FIRMWARE_UPDATE = LinkUnitStatus(4)
    LinkUnitStatus.FIRMWARE_MISMATCH = LinkUnitStatus(5)
    LinkUnitStatus.PENDING = LinkUnitStatus(6)
    LinkUnitStatus._fallback = LinkUnitStatus.UNKNOWN

    # structure
    class LinkUnit(Structure):
        idlType = "cascading.CascadeManager_2_0_2.LinkUnit:1.0.0"
        elements = ["type", "host", "status", "fwVersion"]

        def __init__(self, type=None, host="", status=None, fwVersion=""):
            if type is None:
                type = raritan.rpc.cascading.CascadeManager.LinkUnitType.NETWORK
            if status is None:
                status = raritan.rpc.cascading.CascadeManager.LinkUnitStatus.UNKNOWN
            typecheck.is_enum(type, raritan.rpc.cascading.CascadeManager.LinkUnitType, AssertionError)
            if not typecheck._is_int(host):
                typecheck.is_string(host, AssertionError)
            typecheck.is_enum(status, raritan.rpc.cascading.CascadeManager.LinkUnitStatus, AssertionError)
            if not typecheck._is_int(fwVersion):
                typecheck.is_string(fwVersion, AssertionError)

            self.type = type
            self.host = host
            self.status = status
            self.fwVersion = fwVersion

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                type = raritan.rpc.cascading.CascadeManager.LinkUnitType.decode(json['type']) if 'type' in json or not useDefaults else raritan.rpc.cascading.CascadeManager.LinkUnitType.NETWORK,
                host = json['host'] if 'host' in json or not useDefaults else "",
                status = raritan.rpc.cascading.CascadeManager.LinkUnitStatus.decode(json['status']) if 'status' in json or not useDefaults else raritan.rpc.cascading.CascadeManager.LinkUnitStatus.UNKNOWN,
                fwVersion = json['fwVersion'] if 'fwVersion' in json or not useDefaults else "",
            )
            return obj

        def encode(self):
            json = {}
            json['type'] = raritan.rpc.cascading.CascadeManager.LinkUnitType.encode(self.type)
            json['host'] = str(self.host)
            json['status'] = raritan.rpc.cascading.CascadeManager.LinkUnitStatus.encode(self.status)
            json['fwVersion'] = str(self.fwVersion)
            return json

    # structure
    class Status(Structure):
        idlType = "cascading.CascadeManager_2_0_2.Status:1.0.0"
        elements = ["role", "primaryUnit", "linkUnits"]

        def __init__(self, role=None, primaryUnit="", linkUnits={}):
            if role is None:
                role = raritan.rpc.cascading.CascadeManager.Role.STANDALONE
            typecheck.is_enum(role, raritan.rpc.cascading.CascadeManager.Role, AssertionError)
            if not typecheck._is_int(primaryUnit):
                typecheck.is_string(primaryUnit, AssertionError)

            self.role = role
            self.primaryUnit = primaryUnit
            self.linkUnits = linkUnits

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                role = raritan.rpc.cascading.CascadeManager.Role.decode(json['role']) if 'role' in json or not useDefaults else raritan.rpc.cascading.CascadeManager.Role.STANDALONE,
                primaryUnit = json['primaryUnit'] if 'primaryUnit' in json or not useDefaults else "",
                linkUnits = dict([(
                    elem['key'],
                    raritan.rpc.cascading.CascadeManager.LinkUnit.decode(elem['value'], agent, useDefaults=useDefaults))
                    for elem in json['linkUnits']]) if 'linkUnits' in json or not useDefaults else {},
            )
            return obj

        def encode(self):
            json = {}
            json['role'] = raritan.rpc.cascading.CascadeManager.Role.encode(self.role)
            json['primaryUnit'] = str(self.primaryUnit)
            json['linkUnits'] = [dict(
                key = k,
                value = raritan.rpc.cascading.CascadeManager.LinkUnit.encode(v))
                for k, v in self.linkUnits.items()]
            return json

    # structure
    class LinkPortStatus(Structure):
        idlType = "cascading.CascadeManager_2_0_2.LinkPortStatus:1.0.0"
        elements = ["isSupported", "isLinkDetected", "isLinkingConfirmationNeeded", "connectedNeighborAddr"]

        def __init__(self, isSupported=False, isLinkDetected=False, isLinkingConfirmationNeeded=False, connectedNeighborAddr=""):
            typecheck.is_bool(isSupported, AssertionError)
            typecheck.is_bool(isLinkDetected, AssertionError)
            typecheck.is_bool(isLinkingConfirmationNeeded, AssertionError)
            if not typecheck._is_int(connectedNeighborAddr):
                typecheck.is_string(connectedNeighborAddr, AssertionError)

            self.isSupported = isSupported
            self.isLinkDetected = isLinkDetected
            self.isLinkingConfirmationNeeded = isLinkingConfirmationNeeded
            self.connectedNeighborAddr = connectedNeighborAddr

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                isSupported = json['isSupported'] if 'isSupported' in json or not useDefaults else False,
                isLinkDetected = json['isLinkDetected'] if 'isLinkDetected' in json or not useDefaults else False,
                isLinkingConfirmationNeeded = json['isLinkingConfirmationNeeded'] if 'isLinkingConfirmationNeeded' in json or not useDefaults else False,
                connectedNeighborAddr = json['connectedNeighborAddr'] if 'connectedNeighborAddr' in json or not useDefaults else "",
            )
            return obj

        def encode(self):
            json = {}
            json['isSupported'] = self.isSupported
            json['isLinkDetected'] = self.isLinkDetected
            json['isLinkingConfirmationNeeded'] = self.isLinkingConfirmationNeeded
            json['connectedNeighborAddr'] = str(self.connectedNeighborAddr)
            return json

    # value object
    class RoleChangedEvent(raritan.rpc.idl.Event):
        idlType = "cascading.CascadeManager_2_0_2.RoleChangedEvent:1.0.0"

        def __init__(self, oldRole=None, newRole=None, primaryUnit="", source=None):
            super(raritan.rpc.cascading.CascadeManager.RoleChangedEvent, self).__init__(source)
            if oldRole is None:
                oldRole = raritan.rpc.cascading.CascadeManager.Role.STANDALONE
            if newRole is None:
                newRole = raritan.rpc.cascading.CascadeManager.Role.STANDALONE
            typecheck.is_enum(oldRole, raritan.rpc.cascading.CascadeManager.Role, AssertionError)
            typecheck.is_enum(newRole, raritan.rpc.cascading.CascadeManager.Role, AssertionError)
            if not typecheck._is_int(primaryUnit):
                typecheck.is_string(primaryUnit, AssertionError)

            self.oldRole = oldRole
            self.newRole = newRole
            self.primaryUnit = primaryUnit

        def encode(self):
            json = super(raritan.rpc.cascading.CascadeManager.RoleChangedEvent, self).encode()
            json['oldRole'] = raritan.rpc.cascading.CascadeManager.Role.encode(self.oldRole)
            json['newRole'] = raritan.rpc.cascading.CascadeManager.Role.encode(self.newRole)
            json['primaryUnit'] = str(self.primaryUnit)
            return json

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                oldRole = raritan.rpc.cascading.CascadeManager.Role.decode(json['oldRole']) if 'oldRole' in json or not useDefaults else raritan.rpc.cascading.CascadeManager.Role.STANDALONE,
                newRole = raritan.rpc.cascading.CascadeManager.Role.decode(json['newRole']) if 'newRole' in json or not useDefaults else raritan.rpc.cascading.CascadeManager.Role.STANDALONE,
                primaryUnit = json['primaryUnit'] if 'primaryUnit' in json or not useDefaults else "",
                # for idl.Event
                source = Interface.decode(json['source'], agent) if 'source' in json or not useDefaults else None,
            )
            return obj

        def listElements(self):
            elements = ["oldRole", "newRole", "primaryUnit"]
            elements = elements + super(raritan.rpc.cascading.CascadeManager.RoleChangedEvent, self).listElements()
            return elements

    # value object
    class LinkUnitAddedEvent(raritan.rpc.event.UserEvent):
        idlType = "cascading.CascadeManager_2_0_2.LinkUnitAddedEvent:1.0.0"

        def __init__(self, linkId=0, type=None, host="", actUserName="", actIpAddr="", source=None):
            super(raritan.rpc.cascading.CascadeManager.LinkUnitAddedEvent, self).__init__(actUserName, actIpAddr, source)
            if type is None:
                type = raritan.rpc.cascading.CascadeManager.LinkUnitType.NETWORK
            typecheck.is_int(linkId, AssertionError)
            typecheck.is_enum(type, raritan.rpc.cascading.CascadeManager.LinkUnitType, AssertionError)
            if not typecheck._is_int(host):
                typecheck.is_string(host, AssertionError)

            self.linkId = linkId
            self.type = type
            self.host = host

        def encode(self):
            json = super(raritan.rpc.cascading.CascadeManager.LinkUnitAddedEvent, self).encode()
            json['linkId'] = self.linkId
            json['type'] = raritan.rpc.cascading.CascadeManager.LinkUnitType.encode(self.type)
            json['host'] = str(self.host)
            return json

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                linkId = json['linkId'] if 'linkId' in json or not useDefaults else 0,
                type = raritan.rpc.cascading.CascadeManager.LinkUnitType.decode(json['type']) if 'type' in json or not useDefaults else raritan.rpc.cascading.CascadeManager.LinkUnitType.NETWORK,
                host = json['host'] if 'host' in json or not useDefaults else "",
                # for event.UserEvent
                actUserName = json['actUserName'] if 'actUserName' in json or not useDefaults else "",
                actIpAddr = json['actIpAddr'] if 'actIpAddr' in json or not useDefaults else "",
                # for idl.Event
                source = Interface.decode(json['source'], agent) if 'source' in json or not useDefaults else None,
            )
            return obj

        def listElements(self):
            elements = ["linkId", "type", "host"]
            elements = elements + super(raritan.rpc.cascading.CascadeManager.LinkUnitAddedEvent, self).listElements()
            return elements

    # value object
    class LinkUnitReleasedEvent(raritan.rpc.event.UserEvent):
        idlType = "cascading.CascadeManager_2_0_2.LinkUnitReleasedEvent:1.0.0"

        def __init__(self, linkId=0, type=None, host="", actUserName="", actIpAddr="", source=None):
            super(raritan.rpc.cascading.CascadeManager.LinkUnitReleasedEvent, self).__init__(actUserName, actIpAddr, source)
            if type is None:
                type = raritan.rpc.cascading.CascadeManager.LinkUnitType.NETWORK
            typecheck.is_int(linkId, AssertionError)
            typecheck.is_enum(type, raritan.rpc.cascading.CascadeManager.LinkUnitType, AssertionError)
            if not typecheck._is_int(host):
                typecheck.is_string(host, AssertionError)

            self.linkId = linkId
            self.type = type
            self.host = host

        def encode(self):
            json = super(raritan.rpc.cascading.CascadeManager.LinkUnitReleasedEvent, self).encode()
            json['linkId'] = self.linkId
            json['type'] = raritan.rpc.cascading.CascadeManager.LinkUnitType.encode(self.type)
            json['host'] = str(self.host)
            return json

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                linkId = json['linkId'] if 'linkId' in json or not useDefaults else 0,
                type = raritan.rpc.cascading.CascadeManager.LinkUnitType.decode(json['type']) if 'type' in json or not useDefaults else raritan.rpc.cascading.CascadeManager.LinkUnitType.NETWORK,
                host = json['host'] if 'host' in json or not useDefaults else "",
                # for event.UserEvent
                actUserName = json['actUserName'] if 'actUserName' in json or not useDefaults else "",
                actIpAddr = json['actIpAddr'] if 'actIpAddr' in json or not useDefaults else "",
                # for idl.Event
                source = Interface.decode(json['source'], agent) if 'source' in json or not useDefaults else None,
            )
            return obj

        def listElements(self):
            elements = ["linkId", "type", "host"]
            elements = elements + super(raritan.rpc.cascading.CascadeManager.LinkUnitReleasedEvent, self).listElements()
            return elements

    # value object
    class LinkUnitStatusChangedEvent(raritan.rpc.idl.Event):
        idlType = "cascading.CascadeManager_2_0_2.LinkUnitStatusChangedEvent:1.0.0"

        def __init__(self, linkId=0, type=None, host="", oldStatus=None, newStatus=None, source=None):
            super(raritan.rpc.cascading.CascadeManager.LinkUnitStatusChangedEvent, self).__init__(source)
            if type is None:
                type = raritan.rpc.cascading.CascadeManager.LinkUnitType.NETWORK
            if oldStatus is None:
                oldStatus = raritan.rpc.cascading.CascadeManager.LinkUnitStatus.UNKNOWN
            if newStatus is None:
                newStatus = raritan.rpc.cascading.CascadeManager.LinkUnitStatus.UNKNOWN
            typecheck.is_int(linkId, AssertionError)
            typecheck.is_enum(type, raritan.rpc.cascading.CascadeManager.LinkUnitType, AssertionError)
            if not typecheck._is_int(host):
                typecheck.is_string(host, AssertionError)
            typecheck.is_enum(oldStatus, raritan.rpc.cascading.CascadeManager.LinkUnitStatus, AssertionError)
            typecheck.is_enum(newStatus, raritan.rpc.cascading.CascadeManager.LinkUnitStatus, AssertionError)

            self.linkId = linkId
            self.type = type
            self.host = host
            self.oldStatus = oldStatus
            self.newStatus = newStatus

        def encode(self):
            json = super(raritan.rpc.cascading.CascadeManager.LinkUnitStatusChangedEvent, self).encode()
            json['linkId'] = self.linkId
            json['type'] = raritan.rpc.cascading.CascadeManager.LinkUnitType.encode(self.type)
            json['host'] = str(self.host)
            json['oldStatus'] = raritan.rpc.cascading.CascadeManager.LinkUnitStatus.encode(self.oldStatus)
            json['newStatus'] = raritan.rpc.cascading.CascadeManager.LinkUnitStatus.encode(self.newStatus)
            return json

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                linkId = json['linkId'] if 'linkId' in json or not useDefaults else 0,
                type = raritan.rpc.cascading.CascadeManager.LinkUnitType.decode(json['type']) if 'type' in json or not useDefaults else raritan.rpc.cascading.CascadeManager.LinkUnitType.NETWORK,
                host = json['host'] if 'host' in json or not useDefaults else "",
                oldStatus = raritan.rpc.cascading.CascadeManager.LinkUnitStatus.decode(json['oldStatus']) if 'oldStatus' in json or not useDefaults else raritan.rpc.cascading.CascadeManager.LinkUnitStatus.UNKNOWN,
                newStatus = raritan.rpc.cascading.CascadeManager.LinkUnitStatus.decode(json['newStatus']) if 'newStatus' in json or not useDefaults else raritan.rpc.cascading.CascadeManager.LinkUnitStatus.UNKNOWN,
                # for idl.Event
                source = Interface.decode(json['source'], agent) if 'source' in json or not useDefaults else None,
            )
            return obj

        def listElements(self):
            elements = ["linkId", "type", "host", "oldStatus", "newStatus"]
            elements = elements + super(raritan.rpc.cascading.CascadeManager.LinkUnitStatusChangedEvent, self).listElements()
            return elements

    # value object
    class LinkPortStatusChangedEvent(raritan.rpc.idl.Event):
        idlType = "cascading.CascadeManager_2_0_2.LinkPortStatusChangedEvent:1.0.0"

        def __init__(self, oldStatus=None, newStatus=None, source=None):
            super(raritan.rpc.cascading.CascadeManager.LinkPortStatusChangedEvent, self).__init__(source)
            if oldStatus is None:
                oldStatus = raritan.rpc.cascading.CascadeManager.LinkPortStatus()
            if newStatus is None:
                newStatus = raritan.rpc.cascading.CascadeManager.LinkPortStatus()
            typecheck.is_struct(oldStatus, raritan.rpc.cascading.CascadeManager.LinkPortStatus, AssertionError)
            typecheck.is_struct(newStatus, raritan.rpc.cascading.CascadeManager.LinkPortStatus, AssertionError)

            self.oldStatus = oldStatus
            self.newStatus = newStatus

        def encode(self):
            json = super(raritan.rpc.cascading.CascadeManager.LinkPortStatusChangedEvent, self).encode()
            json['oldStatus'] = raritan.rpc.cascading.CascadeManager.LinkPortStatus.encode(self.oldStatus)
            json['newStatus'] = raritan.rpc.cascading.CascadeManager.LinkPortStatus.encode(self.newStatus)
            return json

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                oldStatus = raritan.rpc.cascading.CascadeManager.LinkPortStatus.decode(json['oldStatus'], agent, useDefaults=useDefaults) if 'oldStatus' in json or not useDefaults else raritan.rpc.cascading.CascadeManager.LinkPortStatus(),
                newStatus = raritan.rpc.cascading.CascadeManager.LinkPortStatus.decode(json['newStatus'], agent, useDefaults=useDefaults) if 'newStatus' in json or not useDefaults else raritan.rpc.cascading.CascadeManager.LinkPortStatus(),
                # for idl.Event
                source = Interface.decode(json['source'], agent) if 'source' in json or not useDefaults else None,
            )
            return obj

        def listElements(self):
            elements = ["oldStatus", "newStatus"]
            elements = elements + super(raritan.rpc.cascading.CascadeManager.LinkPortStatusChangedEvent, self).listElements()
            return elements

    class _getPrimaryUnitSettings(Interface.Method):
        name = 'getPrimaryUnitSettings'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.cascading.CascadeManager.PrimaryUnitSettings.decode(rsp['_ret_'], agent, useDefaults=useDefaults)
            typecheck.is_struct(_ret_, raritan.rpc.cascading.CascadeManager.PrimaryUnitSettings, DecodeException)
            return _ret_

    class _setPrimaryUnitSettings(Interface.Method):
        name = 'setPrimaryUnitSettings'

        @staticmethod
        def encode(primaryUnitSettings):
            typecheck.is_struct(primaryUnitSettings, raritan.rpc.cascading.CascadeManager.PrimaryUnitSettings, AssertionError)
            args = {}
            args['primaryUnitSettings'] = raritan.rpc.cascading.CascadeManager.PrimaryUnitSettings.encode(primaryUnitSettings)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getStatus(Interface.Method):
        name = 'getStatus'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.cascading.CascadeManager.Status.decode(rsp['_ret_'], agent, useDefaults=useDefaults)
            typecheck.is_struct(_ret_, raritan.rpc.cascading.CascadeManager.Status, DecodeException)
            return _ret_

    class _getLinkPortStatus(Interface.Method):
        name = 'getLinkPortStatus'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.cascading.CascadeManager.LinkPortStatus.decode(rsp['_ret_'], agent, useDefaults=useDefaults)
            typecheck.is_struct(_ret_, raritan.rpc.cascading.CascadeManager.LinkPortStatus, DecodeException)
            return _ret_

    class _addLinkUnit(Interface.Method):
        name = 'addLinkUnit'

        @staticmethod
        def encode(linkId, host, login, password, newPassword):
            typecheck.is_int(linkId, AssertionError)
            if not typecheck._is_int(host):
                typecheck.is_string(host, AssertionError)
            if not typecheck._is_int(login):
                typecheck.is_string(login, AssertionError)
            if not typecheck._is_int(password):
                typecheck.is_string(password, AssertionError)
            if not typecheck._is_int(newPassword):
                typecheck.is_string(newPassword, AssertionError)
            args = {}
            args['linkId'] = linkId
            args['host'] = str(host)
            args['login'] = str(login)
            args['password'] = str(password)
            args['newPassword'] = str(newPassword)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _addLinkUnit2(Interface.Method):
        name = 'addLinkUnit2'

        @staticmethod
        def encode(linkId, host, login, password, newPassword, disableStrongPasswordReq):
            typecheck.is_int(linkId, AssertionError)
            if not typecheck._is_int(host):
                typecheck.is_string(host, AssertionError)
            if not typecheck._is_int(login):
                typecheck.is_string(login, AssertionError)
            if not typecheck._is_int(password):
                typecheck.is_string(password, AssertionError)
            if not typecheck._is_int(newPassword):
                typecheck.is_string(newPassword, AssertionError)
            typecheck.is_bool(disableStrongPasswordReq, AssertionError)
            args = {}
            args['linkId'] = linkId
            args['host'] = str(host)
            args['login'] = str(login)
            args['password'] = str(password)
            args['newPassword'] = str(newPassword)
            args['disableStrongPasswordReq'] = disableStrongPasswordReq
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _releaseLinkUnit(Interface.Method):
        name = 'releaseLinkUnit'

        @staticmethod
        def encode(linkId):
            typecheck.is_int(linkId, AssertionError)
            args = {}
            args['linkId'] = linkId
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _requestLink(Interface.Method):
        name = 'requestLink'

        @staticmethod
        def encode(token):
            if not typecheck._is_int(token):
                typecheck.is_string(token, AssertionError)
            args = {}
            args['token'] = str(token)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _finalizeLink(Interface.Method):
        name = 'finalizeLink'

        @staticmethod
        def encode(token):
            if not typecheck._is_int(token):
                typecheck.is_string(token, AssertionError)
            args = {}
            args['token'] = str(token)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            return None

    class _unlink(Interface.Method):
        name = 'unlink'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            return None

    class _getSupportedRoles(Interface.Method):
        name = 'getSupportedRoles'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = [raritan.rpc.cascading.CascadeManager.Role.decode(x0) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_enum(x0, raritan.rpc.cascading.CascadeManager.Role, DecodeException)
            return _ret_

    class _getSupportedLinkUnitTypes(Interface.Method):
        name = 'getSupportedLinkUnitTypes'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = [raritan.rpc.cascading.CascadeManager.LinkUnitType.decode(x0) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_enum(x0, raritan.rpc.cascading.CascadeManager.LinkUnitType, DecodeException)
            return _ret_

    class _addCascadeLinkUnit(Interface.Method):
        name = 'addCascadeLinkUnit'

        @staticmethod
        def encode(linkId, nodeIndex, login, password, positionDependent):
            typecheck.is_int(linkId, AssertionError)
            typecheck.is_int(nodeIndex, AssertionError)
            if not typecheck._is_int(login):
                typecheck.is_string(login, AssertionError)
            if not typecheck._is_int(password):
                typecheck.is_string(password, AssertionError)
            typecheck.is_bool(positionDependent, AssertionError)
            args = {}
            args['linkId'] = linkId
            args['nodeIndex'] = nodeIndex
            args['login'] = str(login)
            args['password'] = str(password)
            args['positionDependent'] = positionDependent
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _addLinkPortLinkUnit(Interface.Method):
        name = 'addLinkPortLinkUnit'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _addSecureSerialLinkUnit(Interface.Method):
        name = 'addSecureSerialLinkUnit'

        @staticmethod
        def encode(linkId, installKey):
            typecheck.is_int(linkId, AssertionError)
            if not typecheck._is_int(installKey):
                typecheck.is_string(installKey, AssertionError)
            args = {}
            args['linkId'] = linkId
            args['installKey'] = str(installKey)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(CascadeManager, self).__init__(target, agent)
        self.getPrimaryUnitSettings = CascadeManager._getPrimaryUnitSettings(self)
        self.setPrimaryUnitSettings = CascadeManager._setPrimaryUnitSettings(self)
        self.getStatus = CascadeManager._getStatus(self)
        self.getLinkPortStatus = CascadeManager._getLinkPortStatus(self)
        self.addLinkUnit = CascadeManager._addLinkUnit(self)
        self.addLinkUnit2 = CascadeManager._addLinkUnit2(self)
        self.releaseLinkUnit = CascadeManager._releaseLinkUnit(self)
        self.requestLink = CascadeManager._requestLink(self)
        self.finalizeLink = CascadeManager._finalizeLink(self)
        self.unlink = CascadeManager._unlink(self)
        self.getSupportedRoles = CascadeManager._getSupportedRoles(self)
        self.getSupportedLinkUnitTypes = CascadeManager._getSupportedLinkUnitTypes(self)
        self.addCascadeLinkUnit = CascadeManager._addCascadeLinkUnit(self)
        self.addLinkPortLinkUnit = CascadeManager._addLinkPortLinkUnit(self)
        self.addSecureSerialLinkUnit = CascadeManager._addSecureSerialLinkUnit(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2025 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "PortFuse.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.idl

import raritan.rpc.portsmodel


# interface
class PortFuse(Interface):
    idlType = "portsmodel.PortFuse:1.0.1"

    # enumeration
    class Status(Enumeration):
        idlType = "portsmodel.PortFuse_1_0_1.Status:1.0.0"
        values = ["UNKNOWN", "TRIPPED", "GOOD"]

    Status.UNKNOWN = Status(0)
    Status.TRIPPED = Status(1)
    Status.GOOD = Status(2)

    # value object
    class StatusChangedEvent(raritan.rpc.idl.Event):
        idlType = "portsmodel.PortFuse_1_0_1.StatusChangedEvent:1.0.0"

        def __init__(self, oldStatus=None, newStatus=None, source=None):
            super(raritan.rpc.portsmodel.PortFuse.StatusChangedEvent, self).__init__(source)
            if oldStatus is None:
                oldStatus = raritan.rpc.portsmodel.PortFuse.Status.UNKNOWN
            if newStatus is None:
                newStatus = raritan.rpc.portsmodel.PortFuse.Status.UNKNOWN
            typecheck.is_enum(oldStatus, raritan.rpc.portsmodel.PortFuse.Status, AssertionError)
            typecheck.is_enum(newStatus, raritan.rpc.portsmodel.PortFuse.Status, AssertionError)

            self.oldStatus = oldStatus
            self.newStatus = newStatus

        def encode(self):
            json = super(raritan.rpc.portsmodel.PortFuse.StatusChangedEvent, self).encode()
            json['oldStatus'] = raritan.rpc.portsmodel.PortFuse.Status.encode(self.oldStatus)
            json['newStatus'] = raritan.rpc.portsmodel.PortFuse.Status.encode(self.newStatus)
            return json

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                oldStatus = raritan.rpc.portsmodel.PortFuse.Status.decode(json['oldStatus']) if 'oldStatus' in json or not useDefaults else raritan.rpc.portsmodel.PortFuse.Status.UNKNOWN,
                newStatus = raritan.rpc.portsmodel.PortFuse.Status.decode(json['newStatus']) if 'newStatus' in json or not useDefaults else raritan.rpc.portsmodel.PortFuse.Status.UNKNOWN,
                # for idl.Event
                source = Interface.decode(json['source'], agent) if 'source' in json or not useDefaults else None,
            )
            return obj

        def listElements(self):
            elements = ["oldStatus", "newStatus"]
            elements = elements + super(raritan.rpc.portsmodel.PortFuse.StatusChangedEvent, self).listElements()
            return elements

    class _getStatus(Interface.Method):
        name = 'getStatus'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.portsmodel.PortFuse.Status.decode(rsp['_ret_'])
            typecheck.is_enum(_ret_, raritan.rpc.portsmodel.PortFuse.Status, DecodeException)
            return _ret_

    class _reset(Interface.Method):
        name = 'reset'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            return None

    class _getTripCount(Interface.Method):
        name = 'getTripCount'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(PortFuse, self).__init__(target, agent)
        self.getStatus = PortFuse._getStatus(self)
        self.reset = PortFuse._reset(self)
        self.getTripCount = PortFuse._getTripCount(self)

#
# Section generated by IdlC from "Port.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.idl

import raritan.rpc.peripheral

import raritan.rpc.portsmodel


# interface
class Port(Interface):
    idlType = "portsmodel.Port:2.0.4"

    NO_ERROR = 0

    ERR_INVALID_PARAM = 1

    ERR_DEVICE_BUSY = 2

    # enumeration
    class DetectionType(Enumeration):
        idlType = "portsmodel.Port_2_0_4.DetectionType:1.0.0"
        values = ["AUTO", "PINNED", "DISABLED"]

    DetectionType.AUTO = DetectionType(0)
    DetectionType.PINNED = DetectionType(1)
    DetectionType.DISABLED = DetectionType(2)

    # enumeration
    class DeviceTypeId(Enumeration):
        idlType = "portsmodel.Port_2_0_4.DeviceTypeId:1.0.0"
        values = ["UNSPECIFIED", "OTHER", "SENSOR_HUB", "BEEPER", "ASSET_STRIP", "POWER_CIM", "GATEWAY_SENSOR"]

    DeviceTypeId.UNSPECIFIED = DeviceTypeId(0)
    DeviceTypeId.OTHER = DeviceTypeId(1)
    DeviceTypeId.SENSOR_HUB = DeviceTypeId(2)
    DeviceTypeId.BEEPER = DeviceTypeId(3)
    DeviceTypeId.ASSET_STRIP = DeviceTypeId(4)
    DeviceTypeId.POWER_CIM = DeviceTypeId(5)
    DeviceTypeId.GATEWAY_SENSOR = DeviceTypeId(6)
    DeviceTypeId._fallback = DeviceTypeId.OTHER

    # structure
    class DeviceTypeWithId(Structure):
        idlType = "portsmodel.Port_2_0_4.DeviceTypeWithId:1.0.0"
        elements = ["id", "type"]

        def __init__(self, id=None, type=""):
            if id is None:
                id = raritan.rpc.portsmodel.Port.DeviceTypeId.UNSPECIFIED
            typecheck.is_enum(id, raritan.rpc.portsmodel.Port.DeviceTypeId, AssertionError)
            if not typecheck._is_int(type):
                typecheck.is_string(type, AssertionError)

            self.id = id
            self.type = type

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                id = raritan.rpc.portsmodel.Port.DeviceTypeId.decode(json['id']) if 'id' in json or not useDefaults else raritan.rpc.portsmodel.Port.DeviceTypeId.UNSPECIFIED,
                type = json['type'] if 'type' in json or not useDefaults else "",
            )
            return obj

        def encode(self):
            json = {}
            json['id'] = raritan.rpc.portsmodel.Port.DeviceTypeId.encode(self.id)
            json['type'] = str(self.type)
            return json

    # structure
    class DetectionMode(Structure):
        idlType = "portsmodel.Port_2_0_4.DetectionMode:1.0.0"
        elements = ["type", "pinnedDeviceType"]

        def __init__(self, type=None, pinnedDeviceType=""):
            if type is None:
                type = raritan.rpc.portsmodel.Port.DetectionType.AUTO
            typecheck.is_enum(type, raritan.rpc.portsmodel.Port.DetectionType, AssertionError)
            if not typecheck._is_int(pinnedDeviceType):
                typecheck.is_string(pinnedDeviceType, AssertionError)

            self.type = type
            self.pinnedDeviceType = pinnedDeviceType

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                type = raritan.rpc.portsmodel.Port.DetectionType.decode(json['type']) if 'type' in json or not useDefaults else raritan.rpc.portsmodel.Port.DetectionType.AUTO,
                pinnedDeviceType = json['pinnedDeviceType'] if 'pinnedDeviceType' in json or not useDefaults else "",
            )
            return obj

        def encode(self):
            json = {}
            json['type'] = raritan.rpc.portsmodel.Port.DetectionType.encode(self.type)
            json['pinnedDeviceType'] = str(self.pinnedDeviceType)
            return json

    # structure
    class Properties(Structure):
        idlType = "portsmodel.Port_2_0_4.Properties:1.0.0"
        elements = ["name", "label", "mode", "detectedDeviceType", "detectedDeviceName", "pinnedDeviceTypeId", "detectedDeviceTypeId", "detectableDeviceTypes", "topoId", "serialId", "portType", "position"]

        def __init__(self, name="", label="", mode=None, detectedDeviceType="", detectedDeviceName="", pinnedDeviceTypeId=None, detectedDeviceTypeId=None, detectableDeviceTypes=[], topoId="", serialId="", portType=None, position=[]):
            if mode is None:
                mode = raritan.rpc.portsmodel.Port.DetectionMode()
            if pinnedDeviceTypeId is None:
                pinnedDeviceTypeId = raritan.rpc.portsmodel.Port.DeviceTypeId.UNSPECIFIED
            if detectedDeviceTypeId is None:
                detectedDeviceTypeId = raritan.rpc.portsmodel.Port.DeviceTypeId.UNSPECIFIED
            if portType is None:
                portType = raritan.rpc.peripheral.PortType.ONBOARD
            if not typecheck._is_int(name):
                typecheck.is_string(name, AssertionError)
            if not typecheck._is_int(label):
                typecheck.is_string(label, AssertionError)
            typecheck.is_struct(mode, raritan.rpc.portsmodel.Port.DetectionMode, AssertionError)
            if not typecheck._is_int(detectedDeviceType):
                typecheck.is_string(detectedDeviceType, AssertionError)
            if not typecheck._is_int(detectedDeviceName):
                typecheck.is_string(detectedDeviceName, AssertionError)
            typecheck.is_enum(pinnedDeviceTypeId, raritan.rpc.portsmodel.Port.DeviceTypeId, AssertionError)
            typecheck.is_enum(detectedDeviceTypeId, raritan.rpc.portsmodel.Port.DeviceTypeId, AssertionError)
            for x0 in detectableDeviceTypes:
                typecheck.is_struct(x0, raritan.rpc.portsmodel.Port.DeviceTypeWithId, AssertionError)
            if not typecheck._is_int(topoId):
                typecheck.is_string(topoId, AssertionError)
            if not typecheck._is_int(serialId):
                typecheck.is_string(serialId, AssertionError)
            typecheck.is_enum(portType, raritan.rpc.peripheral.PortType, AssertionError)
            for x0 in position:
                typecheck.is_struct(x0, raritan.rpc.peripheral.PosElement, AssertionError)

            self.name = name
            self.label = label
            self.mode = mode
            self.detectedDeviceType = detectedDeviceType
            self.detectedDeviceName = detectedDeviceName
            self.pinnedDeviceTypeId = pinnedDeviceTypeId
            self.detectedDeviceTypeId = detectedDeviceTypeId
            self.detectableDeviceTypes = detectableDeviceTypes
            self.topoId = topoId
            self.serialId = serialId
            self.portType = portType
            self.position = position

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                name = json['name'] if 'name' in json or not useDefaults else "",
                label = json['label'] if 'label' in json or not useDefaults else "",
                mode = raritan.rpc.portsmodel.Port.DetectionMode.decode(json['mode'], agent, useDefaults=useDefaults) if 'mode' in json or not useDefaults else raritan.rpc.portsmodel.Port.DetectionMode(),
                detectedDeviceType = json['detectedDeviceType'] if 'detectedDeviceType' in json or not useDefaults else "",
                detectedDeviceName = json['detectedDeviceName'] if 'detectedDeviceName' in json or not useDefaults else "",
                pinnedDeviceTypeId = raritan.rpc.portsmodel.Port.DeviceTypeId.decode(json['pinnedDeviceTypeId']) if 'pinnedDeviceTypeId' in json or not useDefaults else raritan.rpc.portsmodel.Port.DeviceTypeId.UNSPECIFIED,
                detectedDeviceTypeId = raritan.rpc.portsmodel.Port.DeviceTypeId.decode(json['detectedDeviceTypeId']) if 'detectedDeviceTypeId' in json or not useDefaults else raritan.rpc.portsmodel.Port.DeviceTypeId.UNSPECIFIED,
                detectableDeviceTypes = [raritan.rpc.portsmodel.Port.DeviceTypeWithId.decode(x0, agent, useDefaults=useDefaults) for x0 in json['detectableDeviceTypes']] if 'detectableDeviceTypes' in json or not useDefaults else [],
                topoId = json['topoId'] if 'topoId' in json or not useDefaults else "",
                serialId = json['serialId'] if 'serialId' in json or not useDefaults else "",
                portType = raritan.rpc.peripheral.PortType.decode(json['portType']) if 'portType' in json or not useDefaults else raritan.rpc.peripheral.PortType.ONBOARD,
                position = [raritan.rpc.peripheral.PosElement.decode(x0, agent, useDefaults=useDefaults) for x0 in json['position']] if 'position' in json or not useDefaults else [],
            )
            return obj

        def encode(self):
            json = {}
            json['name'] = str(self.name)
            json['label'] = str(self.label)
            json['mode'] = raritan.rpc.portsmodel.Port.DetectionMode.encode(self.mode)
            json['detectedDeviceType'] = str(self.detectedDeviceType)
            json['detectedDeviceName'] = str(self.detectedDeviceName)
            json['pinnedDeviceTypeId'] = raritan.rpc.portsmodel.Port.DeviceTypeId.encode(self.pinnedDeviceTypeId)
            json['detectedDeviceTypeId'] = raritan.rpc.portsmodel.Port.DeviceTypeId.encode(self.detectedDeviceTypeId)
            json['detectableDeviceTypes'] = [raritan.rpc.portsmodel.Port.DeviceTypeWithId.encode(x0) for x0 in self.detectableDeviceTypes]
            json['topoId'] = str(self.topoId)
            json['serialId'] = str(self.serialId)
            json['portType'] = raritan.rpc.peripheral.PortType.encode(self.portType)
            json['position'] = [raritan.rpc.peripheral.PosElement.encode(x0) for x0 in self.position]
            return json

    # value object
    class PropertiesChangedEvent(raritan.rpc.idl.Event):
        idlType = "portsmodel.Port_2_0_4.PropertiesChangedEvent:1.0.0"

        def __init__(self, oldProperties=None, newProperties=None, source=None):
            super(raritan.rpc.portsmodel.Port.PropertiesChangedEvent, self).__init__(source)
            if oldProperties is None:
                oldProperties = raritan.rpc.portsmodel.Port.Properties()
            if newProperties is None:
                newProperties = raritan.rpc.portsmodel.Port.Properties()
            typecheck.is_struct(oldProperties, raritan.rpc.portsmodel.Port.Properties, AssertionError)
            typecheck.is_struct(newProperties, raritan.rpc.portsmodel.Port.Properties, AssertionError)

            self.oldProperties = oldProperties
            self.newProperties = newProperties

        def encode(self):
            json = super(raritan.rpc.portsmodel.Port.PropertiesChangedEvent, self).encode()
            json['oldProperties'] = raritan.rpc.portsmodel.Port.Properties.encode(self.oldProperties)
            json['newProperties'] = raritan.rpc.portsmodel.Port.Properties.encode(self.newProperties)
            return json

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                oldProperties = raritan.rpc.portsmodel.Port.Properties.decode(json['oldProperties'], agent, useDefaults=useDefaults) if 'oldProperties' in json or not useDefaults else raritan.rpc.portsmodel.Port.Properties(),
                newProperties = raritan.rpc.portsmodel.Port.Properties.decode(json['newProperties'], agent, useDefaults=useDefaults) if 'newProperties' in json or not useDefaults else raritan.rpc.portsmodel.Port.Properties(),
                # for idl.Event
                source = Interface.decode(json['source'], agent) if 'source' in json or not useDefaults else None,
            )
            return obj

        def listElements(self):
            elements = ["oldProperties", "newProperties"]
            elements = elements + super(raritan.rpc.portsmodel.Port.PropertiesChangedEvent, self).listElements()
            return elements

    # value object
    class DeviceChangedEvent(raritan.rpc.idl.Event):
        idlType = "portsmodel.Port_2_0_4.DeviceChangedEvent:1.0.0"

        def __init__(self, oldDevice=None, newDevice=None, source=None):
            super(raritan.rpc.portsmodel.Port.DeviceChangedEvent, self).__init__(source)
            typecheck.is_remote_obj(oldDevice, AssertionError)
            typecheck.is_remote_obj(newDevice, AssertionError)

            self.oldDevice = oldDevice
            self.newDevice = newDevice

        def encode(self):
            json = super(raritan.rpc.portsmodel.Port.DeviceChangedEvent, self).encode()
            json['oldDevice'] = Interface.encode(self.oldDevice)
            json['newDevice'] = Interface.encode(self.newDevice)
            return json

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                oldDevice = Interface.decode(json['oldDevice'], agent) if 'oldDevice' in json or not useDefaults else None,
                newDevice = Interface.decode(json['newDevice'], agent) if 'newDevice' in json or not useDefaults else None,
                # for idl.Event
                source = Interface.decode(json['source'], agent) if 'source' in json or not useDefaults else None,
            )
            return obj

        def listElements(self):
            elements = ["oldDevice", "newDevice"]
            elements = elements + super(raritan.rpc.portsmodel.Port.DeviceChangedEvent, self).listElements()
            return elements

    class _getProperties(Interface.Method):
        name = 'getProperties'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.portsmodel.Port.Properties.decode(rsp['_ret_'], agent, useDefaults=useDefaults)
            typecheck.is_struct(_ret_, raritan.rpc.portsmodel.Port.Properties, DecodeException)
            return _ret_

    class _setName(Interface.Method):
        name = 'setName'

        @staticmethod
        def encode(name):
            if not typecheck._is_int(name):
                typecheck.is_string(name, AssertionError)
            args = {}
            args['name'] = str(name)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            return None

    class _setDetectionMode(Interface.Method):
        name = 'setDetectionMode'

        @staticmethod
        def encode(mode):
            typecheck.is_struct(mode, raritan.rpc.portsmodel.Port.DetectionMode, AssertionError)
            args = {}
            args['mode'] = raritan.rpc.portsmodel.Port.DetectionMode.encode(mode)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getDetectableDevices(Interface.Method):
        name = 'getDetectableDevices'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = [x0 for x0 in rsp['_ret_']]
            for x0 in _ret_:
                if not typecheck._is_int(x0):
                    typecheck.is_string(x0, DecodeException)
            return _ret_

    class _getDevice(Interface.Method):
        name = 'getDevice'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = Interface.decode(rsp['_ret_'], agent)
            typecheck.is_remote_obj(_ret_, DecodeException)
            return _ret_

    class _getDeviceConfig(Interface.Method):
        name = 'getDeviceConfig'

        @staticmethod
        def encode(deviceType):
            if not typecheck._is_int(deviceType):
                typecheck.is_string(deviceType, AssertionError)
            args = {}
            args['deviceType'] = str(deviceType)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = Interface.decode(rsp['_ret_'], agent)
            typecheck.is_remote_obj(_ret_, DecodeException)
            return _ret_

    class _getFuse(Interface.Method):
        name = 'getFuse'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = Interface.decode(rsp['_ret_'], agent)
            typecheck.is_interface(_ret_, raritan.rpc.portsmodel.PortFuse, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(Port, self).__init__(target, agent)
        self.getProperties = Port._getProperties(self)
        self.setName = Port._setName(self)
        self.setDetectionMode = Port._setDetectionMode(self)
        self.getDetectableDevices = Port._getDetectableDevices(self)
        self.getDevice = Port._getDevice(self)
        self.getDeviceConfig = Port._getDeviceConfig(self)
        self.getFuse = Port._getFuse(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2025 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "DisplayControl.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.display

import raritan.rpc.event


# interface
class DisplayControl(Interface):
    idlType = "display.DisplayControl:2.0.0"

    SUCCESS = 0

    ERR_INVALID_VALUE = 1

    # structure
    class Info(Structure):
        idlType = "display.DisplayControl_2_0_0.Info:1.0.0"
        elements = ["fwAppVersion", "fwBootVersion", "orientation", "width", "height", "versionAvailable"]

        def __init__(self, fwAppVersion=0, fwBootVersion=0, orientation=0, width=0, height=0, versionAvailable=False):
            typecheck.is_int(fwAppVersion, AssertionError)
            typecheck.is_int(fwBootVersion, AssertionError)
            typecheck.is_int(orientation, AssertionError)
            typecheck.is_int(width, AssertionError)
            typecheck.is_int(height, AssertionError)
            typecheck.is_bool(versionAvailable, AssertionError)

            self.fwAppVersion = fwAppVersion
            self.fwBootVersion = fwBootVersion
            self.orientation = orientation
            self.width = width
            self.height = height
            self.versionAvailable = versionAvailable

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                fwAppVersion = json['fwAppVersion'] if 'fwAppVersion' in json or not useDefaults else 0,
                fwBootVersion = json['fwBootVersion'] if 'fwBootVersion' in json or not useDefaults else 0,
                orientation = json['orientation'] if 'orientation' in json or not useDefaults else 0,
                width = json['width'] if 'width' in json or not useDefaults else 0,
                height = json['height'] if 'height' in json or not useDefaults else 0,
                versionAvailable = json['versionAvailable'] if 'versionAvailable' in json or not useDefaults else False,
            )
            return obj

        def encode(self):
            json = {}
            json['fwAppVersion'] = self.fwAppVersion
            json['fwBootVersion'] = self.fwBootVersion
            json['orientation'] = self.orientation
            json['width'] = self.width
            json['height'] = self.height
            json['versionAvailable'] = self.versionAvailable
            return json

    # structure
    class DefaultViewItem(Structure):
        idlType = "display.DisplayControl_2_0_0.DefaultViewItem:1.0.0"
        elements = ["id", "description"]

        def __init__(self, id="", description=""):
            if not typecheck._is_int(id):
                typecheck.is_string(id, AssertionError)
            if not typecheck._is_int(description):
                typecheck.is_string(description, AssertionError)

            self.id = id
            self.description = description

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                id = json['id'] if 'id' in json or not useDefaults else "",
                description = json['description'] if 'description' in json or not useDefaults else "",
            )
            return obj

        def encode(self):
            json = {}
            json['id'] = str(self.id)
            json['description'] = str(self.description)
            return json

    # structure
    class Settings(Structure):
        idlType = "display.DisplayControl_2_0_0.Settings:1.0.0"
        elements = ["defaultView", "backlightTimeout"]

        def __init__(self, defaultView="", backlightTimeout=0):
            if not typecheck._is_int(defaultView):
                typecheck.is_string(defaultView, AssertionError)
            typecheck.is_int(backlightTimeout, AssertionError)

            self.defaultView = defaultView
            self.backlightTimeout = backlightTimeout

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                defaultView = json['defaultView'] if 'defaultView' in json or not useDefaults else "",
                backlightTimeout = json['backlightTimeout'] if 'backlightTimeout' in json or not useDefaults else 0,
            )
            return obj

        def encode(self):
            json = {}
            json['defaultView'] = str(self.defaultView)
            json['backlightTimeout'] = self.backlightTimeout
            return json

    # value object
    class SettingsChangedEvent(raritan.rpc.event.UserEvent):
        idlType = "display.DisplayControl_2_0_0.SettingsChangedEvent:1.0.0"

        def __init__(self, newSettings=None, actUserName="", actIpAddr="", source=None):
            super(raritan.rpc.display.DisplayControl.SettingsChangedEvent, self).__init__(actUserName, actIpAddr, source)
            if newSettings is None:
                newSettings = raritan.rpc.display.DisplayControl.Settings()
            typecheck.is_struct(newSettings, raritan.rpc.display.DisplayControl.Settings, AssertionError)

            self.newSettings = newSettings

        def encode(self):
            json = super(raritan.rpc.display.DisplayControl.SettingsChangedEvent, self).encode()
            json['newSettings'] = raritan.rpc.display.DisplayControl.Settings.encode(self.newSettings)
            return json

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                newSettings = raritan.rpc.display.DisplayControl.Settings.decode(json['newSettings'], agent, useDefaults=useDefaults) if 'newSettings' in json or not useDefaults else raritan.rpc.display.DisplayControl.Settings(),
                # for event.UserEvent
                actUserName = json['actUserName'] if 'actUserName' in json or not useDefaults else "",
                actIpAddr = json['actIpAddr'] if 'actIpAddr' in json or not useDefaults else "",
                # for idl.Event
                source = Interface.decode(json['source'], agent) if 'source' in json or not useDefaults else None,
            )
            return obj

        def listElements(self):
            elements = ["newSettings"]
            elements = elements + super(raritan.rpc.display.DisplayControl.SettingsChangedEvent, self).listElements()
            return elements

    class _getAvailableDefaultViews(Interface.Method):
        name = 'getAvailableDefaultViews'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = [raritan.rpc.display.DisplayControl.DefaultViewItem.decode(x0, agent, useDefaults=useDefaults) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.display.DisplayControl.DefaultViewItem, DecodeException)
            return _ret_

    class _getSettings(Interface.Method):
        name = 'getSettings'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.display.DisplayControl.Settings.decode(rsp['_ret_'], agent, useDefaults=useDefaults)
            typecheck.is_struct(_ret_, raritan.rpc.display.DisplayControl.Settings, DecodeException)
            return _ret_

    class _setSettings(Interface.Method):
        name = 'setSettings'

        @staticmethod
        def encode(settings):
            typecheck.is_struct(settings, raritan.rpc.display.DisplayControl.Settings, AssertionError)
            args = {}
            args['settings'] = raritan.rpc.display.DisplayControl.Settings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getInfo(Interface.Method):
        name = 'getInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.display.DisplayControl.Info.decode(rsp['_ret_'], agent, useDefaults=useDefaults)
            typecheck.is_struct(_ret_, raritan.rpc.display.DisplayControl.Info, DecodeException)
            return _ret_

    class _showSmiley(Interface.Method):
        name = 'showSmiley'

        @staticmethod
        def encode(on, happy, msg):
            typecheck.is_bool(on, AssertionError)
            typecheck.is_bool(happy, AssertionError)
            if not typecheck._is_int(msg):
                typecheck.is_string(msg, AssertionError)
            args = {}
            args['on'] = on
            args['happy'] = happy
            args['msg'] = str(msg)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            return None
    def __init__(self, target, agent):
        super(DisplayControl, self).__init__(target, agent)
        self.getAvailableDefaultViews = DisplayControl._getAvailableDefaultViews(self)
        self.getSettings = DisplayControl._getSettings(self)
        self.setSettings = DisplayControl._setSettings(self)
        self.getInfo = DisplayControl._getInfo(self)
        self.showSmiley = DisplayControl._showSmiley(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2022 Raritan Inc. All rights reserved.

# Minimalistic implementation of the http.client module for Python 2 compatibility

import urllib2

class HTTPConnection:
    def __init__(self, host, timeout):
        self.scheme = "http"
        self.host = host
        self.timeout = timeout
        self.opener = urllib2.OpenerDirector()
        self.opener.add_handler(urllib2.HTTPHandler())
        self.response = None

    def request(self, method, path, body, headers):
        url = "%s://%s%s" % (self.scheme, self.host, path)
        # Suppress Bandit message: URL scheme is known to be "http" or "https".
        request = urllib2.Request(url, data=body, headers=headers) # nosec B310
        self.response = self.opener.open(request, timeout=self.timeout)

    def getresponse(self):
        response = self.response
        self.response = None
        return response

class HTTPSConnection(HTTPConnection):
    def __init__(self, host, timeout, context):
        HTTPConnection.__init__(self, host, timeout)
        self.scheme = "https"
        try:
            self.opener.add_handler(urllib2.HTTPSHandler(context=context))
        except TypeError:
            # Python < 2.7.9
            self.opener.add_handler(urllib2.HTTPSHandler())

class RemoteDisconnected(Exception):
    pass



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2025 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "Log.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.logging


# structure
class LogInfo(Structure):
    idlType = "logging.LogInfo:1.0.0"
    elements = ["creationTime", "idFirst", "idNext"]

    def __init__(self, creationTime=0, idFirst=0, idNext=0):
        typecheck.is_long(creationTime, AssertionError)
        typecheck.is_int(idFirst, AssertionError)
        typecheck.is_int(idNext, AssertionError)

        self.creationTime = creationTime
        self.idFirst = idFirst
        self.idNext = idNext

    @classmethod
    def decode(cls, json, agent, useDefaults=False):
        obj = cls(
            creationTime = int(json['creationTime']) if 'creationTime' in json or not useDefaults else 0,
            idFirst = json['idFirst'] if 'idFirst' in json or not useDefaults else 0,
            idNext = json['idNext'] if 'idNext' in json or not useDefaults else 0,
        )
        return obj

    def encode(self):
        json = {}
        json['creationTime'] = self.creationTime
        json['idFirst'] = self.idFirst
        json['idNext'] = self.idNext
        return json

# structure
class LogEntry(Structure):
    idlType = "logging.LogEntry:1.0.0"
    elements = ["id", "timestamp", "eventClass", "message"]

    def __init__(self, id=0, timestamp=None, eventClass="", message=""):
        typecheck.is_int(id, AssertionError)
        typecheck.is_time(timestamp, AssertionError)
        if not typecheck._is_int(eventClass):
            typecheck.is_string(eventClass, AssertionError)
        if not typecheck._is_int(message):
            typecheck.is_string(message, AssertionError)

        self.id = id
        self.timestamp = timestamp
        self.eventClass = eventClass
        self.message = message

    @classmethod
    def decode(cls, json, agent, useDefaults=False):
        obj = cls(
            id = json['id'] if 'id' in json or not useDefaults else 0,
            timestamp = raritan.rpc.Time.decode(json['timestamp']) if 'timestamp' in json or not useDefaults else None,
            eventClass = json['eventClass'] if 'eventClass' in json or not useDefaults else "",
            message = json['message'] if 'message' in json or not useDefaults else "",
        )
        return obj

    def encode(self):
        json = {}
        json['id'] = self.id
        json['timestamp'] = raritan.rpc.Time.encode(self.timestamp)
        json['eventClass'] = str(self.eventClass)
        json['message'] = str(self.message)
        return json

# structure
class LogChunk(Structure):
    idlType = "logging.LogChunk:1.0.0"
    elements = ["logCreationTime", "idFirst", "allEntryCnt", "selEntries"]

    def __init__(self, logCreationTime=0, idFirst=0, allEntryCnt=0, selEntries=[]):
        typecheck.is_long(logCreationTime, AssertionError)
        typecheck.is_int(idFirst, AssertionError)
        typecheck.is_int(allEntryCnt, AssertionError)
        for x0 in selEntries:
            typecheck.is_struct(x0, raritan.rpc.logging.LogEntry, AssertionError)

        self.logCreationTime = logCreationTime
        self.idFirst = idFirst
        self.allEntryCnt = allEntryCnt
        self.selEntries = selEntries

    @classmethod
    def decode(cls, json, agent, useDefaults=False):
        obj = cls(
            logCreationTime = int(json['logCreationTime']) if 'logCreationTime' in json or not useDefaults else 0,
            idFirst = json['idFirst'] if 'idFirst' in json or not useDefaults else 0,
            allEntryCnt = json['allEntryCnt'] if 'allEntryCnt' in json or not useDefaults else 0,
            selEntries = [raritan.rpc.logging.LogEntry.decode(x0, agent, useDefaults=useDefaults) for x0 in json['selEntries']] if 'selEntries' in json or not useDefaults else [],
        )
        return obj

    def encode(self):
        json = {}
        json['logCreationTime'] = self.logCreationTime
        json['idFirst'] = self.idFirst
        json['allEntryCnt'] = self.allEntryCnt
        json['selEntries'] = [raritan.rpc.logging.LogEntry.encode(x0) for x0 in self.selEntries]
        return json

# enumeration
class RangeDirection(Enumeration):
    idlType = "logging.RangeDirection:1.0.0"
    values = ["FORWARD", "BACKWARD"]

RangeDirection.FORWARD = RangeDirection(0)
RangeDirection.BACKWARD = RangeDirection(1)

#
# Section generated by IdlC from "DebugLog.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.logging


# interface
class DebugLog(Interface):
    idlType = "logging.DebugLog:2.0.0"

    class _clear(Interface.Method):
        name = 'clear'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            return None

    class _getInfo(Interface.Method):
        name = 'getInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.logging.LogInfo.decode(rsp['_ret_'], agent, useDefaults=useDefaults)
            typecheck.is_struct(_ret_, raritan.rpc.logging.LogInfo, DecodeException)
            return _ret_

    class _getChunk(Interface.Method):
        name = 'getChunk'

        @staticmethod
        def encode(refId, count, direction):
            typecheck.is_int(refId, AssertionError)
            typecheck.is_int(count, AssertionError)
            typecheck.is_enum(direction, raritan.rpc.logging.RangeDirection, AssertionError)
            args = {}
            args['refId'] = refId
            args['count'] = count
            args['direction'] = raritan.rpc.logging.RangeDirection.encode(direction)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.logging.LogChunk.decode(rsp['_ret_'], agent, useDefaults=useDefaults)
            typecheck.is_struct(_ret_, raritan.rpc.logging.LogChunk, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(DebugLog, self).__init__(target, agent)
        self.clear = DebugLog._clear(self)
        self.getInfo = DebugLog._getInfo(self)
        self.getChunk = DebugLog._getChunk(self)

#
# Section generated by IdlC from "EventLog.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.event

import raritan.rpc.logging


# value object
class EventLogClearedEvent(raritan.rpc.event.UserEvent):
    idlType = "logging.EventLogClearedEvent:1.0.0"

    def __init__(self, actUserName="", actIpAddr="", source=None):
        super(raritan.rpc.logging.EventLogClearedEvent, self).__init__(actUserName, actIpAddr, source)

    def encode(self):
        json = super(raritan.rpc.logging.EventLogClearedEvent, self).encode()
        return json

    @classmethod
    def decode(cls, json, agent, useDefaults=False):
        obj = cls(
            # for event.UserEvent
            actUserName = json['actUserName'] if 'actUserName' in json or not useDefaults else "",
            actIpAddr = json['actIpAddr'] if 'actIpAddr' in json or not useDefaults else "",
            # for idl.Event
            source = Interface.decode(json['source'], agent) if 'source' in json or not useDefaults else None,
        )
        return obj

    def listElements(self):
        elements = []
        elements = elements + super(raritan.rpc.logging.EventLogClearedEvent, self).listElements()
        return elements

# interface
class EventLog(Interface):
    idlType = "logging.EventLog:2.0.0"

    class _clear(Interface.Method):
        name = 'clear'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            return None

    class _getInfo(Interface.Method):
        name = 'getInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.logging.LogInfo.decode(rsp['_ret_'], agent, useDefaults=useDefaults)
            typecheck.is_struct(_ret_, raritan.rpc.logging.LogInfo, DecodeException)
            return _ret_

    class _getChunk(Interface.Method):
        name = 'getChunk'

        @staticmethod
        def encode(refId, count, direction, categories):
            typecheck.is_int(refId, AssertionError)
            typecheck.is_int(count, AssertionError)
            typecheck.is_enum(direction, raritan.rpc.logging.RangeDirection, AssertionError)
            for x0 in categories:
                if not typecheck._is_int(x0):
                    typecheck.is_string(x0, AssertionError)
            args = {}
            args['refId'] = refId
            args['count'] = count
            args['direction'] = raritan.rpc.logging.RangeDirection.encode(direction)
            args['categories'] = [str(x0) for x0 in categories]
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.logging.LogChunk.decode(rsp['_ret_'], agent, useDefaults=useDefaults)
            typecheck.is_struct(_ret_, raritan.rpc.logging.LogChunk, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(EventLog, self).__init__(target, agent)
        self.clear = EventLog._clear(self)
        self.getInfo = EventLog._getInfo(self)
        self.getChunk = EventLog._getChunk(self)

#
# Section generated by IdlC from "WlanLog.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.logging


# interface
class WlanLog(Interface):
    idlType = "logging.WlanLog:1.0.0"

    class _clear(Interface.Method):
        name = 'clear'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            return None

    class _getInfo(Interface.Method):
        name = 'getInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.logging.LogInfo.decode(rsp['_ret_'], agent, useDefaults=useDefaults)
            typecheck.is_struct(_ret_, raritan.rpc.logging.LogInfo, DecodeException)
            return _ret_

    class _getChunk(Interface.Method):
        name = 'getChunk'

        @staticmethod
        def encode(refId, count, direction):
            typecheck.is_int(refId, AssertionError)
            typecheck.is_int(count, AssertionError)
            typecheck.is_enum(direction, raritan.rpc.logging.RangeDirection, AssertionError)
            args = {}
            args['refId'] = refId
            args['count'] = count
            args['direction'] = raritan.rpc.logging.RangeDirection.encode(direction)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.logging.LogChunk.decode(rsp['_ret_'], agent, useDefaults=useDefaults)
            typecheck.is_struct(_ret_, raritan.rpc.logging.LogChunk, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(WlanLog, self).__init__(target, agent)
        self.clear = WlanLog._clear(self)
        self.getInfo = WlanLog._getInfo(self)
        self.getChunk = WlanLog._getChunk(self)

# from raritan/rpc/logging/__extend__.py
import datetime
import raritan.rpc.logging

class LogHelper():

    error_tag = "[ERR]"
    warning_tag = "[WRN]"

    def __init__(self, agent, logtype):
        self.agent = agent
        self.logtype = logtype
        entries, self.nextid, self.creationtime = self.get_entries(self.agent, self.logtype)
        self.last_entry = entries[-1] if entries else None

    def get_next_entries(self):
        entries, self.nextid, creationtime = self.get_entries(self.agent, self.logtype, self.nextid, self.creationtime)
        if creationtime != self.creationtime:
            self.creationtime = creationtime
            if self.last_entry:
                # this is needed for following the event log over logd restarts;
                # the restart resets chunk timestamps but doesn't reset id numbering, so we'll get the whole log again
                last_entry_index = next((i for i in reversed(range(len(entries))) if entries[i] == self.last_entry), None)
                if last_entry_index is not None:
                    entries = entries[last_entry_index+1:]
        if entries:
            self.last_entry = entries[-1]
        return entries

    @staticmethod
    def log_proxy(agent, log_type):
        if log_type == "debug":
            return raritan.rpc.logging.DebugLog("/debuglog", agent)
        elif log_type == "event":
            return raritan.rpc.logging.EventLog("/eventlog", agent)
        elif log_type == "wlan":
            return raritan.rpc.logging.WlanLog("/wlanlog", agent)
        else:
            raise ValueError("Not a log type: %s" % log_type)

    @staticmethod
    def format_entry(entry, tz=None):
        timefmt = "%m/%d/%Y %H:%M:%S %Z"
        timestr = entry.timestamp.replace(tzinfo=datetime.timezone.utc).astimezone(tz).strftime(timefmt)
        return "[%s]:%s" % (timestr, entry.message)

    @classmethod
    def get_entries(cls, agent, log_type, ref_id=None, creation_time=None):
        log = cls.log_proxy(agent, log_type)
        info = log.getInfo()
        # remember most recent entry at this point,
        # ignore entries logged after it to circumvent infinite loop
        next_id = info.idNext
        if ref_id is None:
            ref_id = info.idFirst
        if creation_time is None:
            creation_time = info.creationTime
        entries = []

        while(True):
            # count -1 means all entries of the chunk
            args = [ref_id, -1, raritan.rpc.logging.RangeDirection.FORWARD]
            if log_type == "event":
                # argument for event categories filtering required
                args.append([])
            chunk = log.getChunk(*args)
            if chunk.logCreationTime != creation_time:
                # log timestamp was reset, begin from start
                info = log.getInfo()
                next_id = info.idNext
                ref_id = info.idFirst
                creation_time = info.creationTime
                continue
            if chunk.allEntryCnt == 0:
                # reached end of log
                break
            if chunk.idFirst + chunk.allEntryCnt >= next_id:
                # chunk contains our most recent entry
                entries += [entry for entry in chunk.selEntries if entry.id < next_id]
                ref_id = next_id
                break
            entries += chunk.selEntries
            ref_id = chunk.idFirst + chunk.allEntryCnt

        return entries, ref_id, creation_time

    @classmethod
    def filter_entries(cls, entries, search_strings=None):
        if not search_strings:
            search_strings = [cls.error_tag, cls.warning_tag]
        return [entry for entry in entries if any(string in entry.message for string in search_strings)]



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2025 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "AuthManager.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.auth


# enumeration
class Type(Enumeration):
    idlType = "auth.Type:1.0.0"
    values = ["LOCAL", "RADIUS", "KERBEROS", "TACACS_PLUS", "LDAP"]

Type.LOCAL = Type(0)
Type.RADIUS = Type(1)
Type.KERBEROS = Type(2)
Type.TACACS_PLUS = Type(3)
Type.LDAP = Type(4)

# structure
class Policy(Structure):
    idlType = "auth.Policy:1.0.0"
    elements = ["type", "useLocalIfRemoteFailed"]

    def __init__(self, type=None, useLocalIfRemoteFailed=False):
        if type is None:
            type = raritan.rpc.auth.Type.LOCAL
        typecheck.is_enum(type, raritan.rpc.auth.Type, AssertionError)
        typecheck.is_bool(useLocalIfRemoteFailed, AssertionError)

        self.type = type
        self.useLocalIfRemoteFailed = useLocalIfRemoteFailed

    @classmethod
    def decode(cls, json, agent, useDefaults=False):
        obj = cls(
            type = raritan.rpc.auth.Type.decode(json['type']) if 'type' in json or not useDefaults else raritan.rpc.auth.Type.LOCAL,
            useLocalIfRemoteFailed = json['useLocalIfRemoteFailed'] if 'useLocalIfRemoteFailed' in json or not useDefaults else False,
        )
        return obj

    def encode(self):
        json = {}
        json['type'] = raritan.rpc.auth.Type.encode(self.type)
        json['useLocalIfRemoteFailed'] = self.useLocalIfRemoteFailed
        return json

# interface
class AuthManager(Interface):
    idlType = "auth.AuthManager:1.0.0"

    ERR_UNSUPPORTED_TYPE = 1

    class _getPolicy(Interface.Method):
        name = 'getPolicy'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = raritan.rpc.auth.Policy.decode(rsp['_ret_'], agent, useDefaults=useDefaults)
            typecheck.is_struct(_ret_, raritan.rpc.auth.Policy, DecodeException)
            return _ret_

    class _setPolicy(Interface.Method):
        name = 'setPolicy'

        @staticmethod
        def encode(p):
            typecheck.is_struct(p, raritan.rpc.auth.Policy, AssertionError)
            args = {}
            args['p'] = raritan.rpc.auth.Policy.encode(p)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(AuthManager, self).__init__(target, agent)
        self.getPolicy = AuthManager._getPolicy(self)
        self.setPolicy = AuthManager._setPolicy(self)

#
# Section generated by IdlC from "LdapManager.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.auth


# interface
class LdapManager(Interface):
    idlType = "auth.LdapManager:5.0.0"

    # enumeration
    class ServerType(Enumeration):
        idlType = "auth.LdapManager_5_0_0.ServerType:1.0.0"
        values = ["ACTIVE_DIRECTORY", "OPEN_LDAP"]

    ServerType.ACTIVE_DIRECTORY = ServerType(0)
    ServerType.OPEN_LDAP = ServerType(1)

    # enumeration
    class SecurityProtocol(Enumeration):
        idlType = "auth.LdapManager_5_0_0.SecurityProtocol:1.0.0"
        values = ["SEC_PROTO_NONE", "SEC_PROTO_SSL", "SEC_PROTO_STARTTLS"]

    SecurityProtocol.SEC_PROTO_NONE = SecurityProtocol(0)
    SecurityProtocol.SEC_PROTO_SSL = SecurityProtocol(1)
    SecurityProtocol.SEC_PROTO_STARTTLS = SecurityProtocol(2)

    # structure
    class ServerSettings(Structure):
        idlType = "auth.LdapManager_5_0_0.ServerSettings:1.0.0"
        elements = ["server", "type", "secProto", "port", "sslPort", "forceTrustedCert", "allowOffTimeRangeCerts", "certificate", "adsDomain", "useAnonymousBind", "bindDN", "bindPwd", "searchBaseDN", "loginNameAttr", "userEntryObjClass", "userSearchFilter", "groupInfoInUserEntry", "supportNestedGroups", "groupMemberAttr", "groupEntryObjClass", "groupSearchFilter"]

        def __init__(self, server="", type=None, secProto=None, port=0, sslPort=0, forceTrustedCert=False, allowOffTimeRangeCerts=False, certificate="", adsDomain="", useAnonymousBind=False, bindDN="", bindPwd="", searchBaseDN="", loginNameAttr="", userEntryObjClass="", userSearchFilter="", groupInfoInUserEntry=False, supportNestedGroups=False, groupMemberAttr="", groupEntryObjClass="", groupSearchFilter=""):
            if type is None:
                type = raritan.rpc.auth.LdapManager.ServerType.ACTIVE_DIRECTORY
            if secProto is None:
                secProto = raritan.rpc.auth.LdapManager.SecurityProtocol.SEC_PROTO_NONE
            if not typecheck._is_int(server):
                typecheck.is_string(server, AssertionError)
            typecheck.is_enum(type, raritan.rpc.auth.LdapManager.ServerType, AssertionError)
            typecheck.is_enum(secProto, raritan.rpc.auth.LdapManager.SecurityProtocol, AssertionError)
            typecheck.is_int(port, AssertionError)
            typecheck.is_int(sslPort, AssertionError)
            typecheck.is_bool(forceTrustedCert, AssertionError)
            typecheck.is_bool(allowOffTimeRangeCerts, AssertionError)
            if not typecheck._is_int(certificate):
                typecheck.is_string(certificate, AssertionError)
            if not typecheck._is_int(adsDomain):
                typecheck.is_string(adsDomain, AssertionError)
            typecheck.is_bool(useAnonymousBind, AssertionError)
            if not typecheck._is_int(bindDN):
                typecheck.is_string(bindDN, AssertionError)
            if not typecheck._is_int(bindPwd):
                typecheck.is_string(bindPwd, AssertionError)
            if not typecheck._is_int(searchBaseDN):
                typecheck.is_string(searchBaseDN, AssertionError)
            if not typecheck._is_int(loginNameAttr):
                typecheck.is_string(loginNameAttr, AssertionError)
            if not typecheck._is_int(userEntryObjClass):
                typecheck.is_string(userEntryObjClass, AssertionError)
            if not typecheck._is_int(userSearchFilter):
                typecheck.is_string(userSearchFilter, AssertionError)
            typecheck.is_bool(groupInfoInUserEntry, AssertionError)
            typecheck.is_bool(supportNestedGroups, AssertionError)
            if not typecheck._is_int(groupMemberAttr):
                typecheck.is_string(groupMemberAttr, AssertionError)
            if not typecheck._is_int(groupEntryObjClass):
                typecheck.is_string(groupEntryObjClass, AssertionError)
            if not typecheck._is_int(groupSearchFilter):
                typecheck.is_string(groupSearchFilter, AssertionError)

            self.server = server
            self.type = type
            self.secProto = secProto
            self.port = port
            self.sslPort = sslPort
            self.forceTrustedCert = forceTrustedCert
            self.allowOffTimeRangeCerts = allowOffTimeRangeCerts
            self.certificate = certificate
            self.adsDomain = adsDomain
            self.useAnonymousBind = useAnonymousBind
            self.bindDN = bindDN
            self.bindPwd = bindPwd
            self.searchBaseDN = searchBaseDN
            self.loginNameAttr = loginNameAttr
            self.userEntryObjClass = userEntryObjClass
            self.userSearchFilter = userSearchFilter
            self.groupInfoInUserEntry = groupInfoInUserEntry
            self.supportNestedGroups = supportNestedGroups
            self.groupMemberAttr = groupMemberAttr
            self.groupEntryObjClass = groupEntryObjClass
            self.groupSearchFilter = groupSearchFilter

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                server = json['server'] if 'server' in json or not useDefaults else "",
                type = raritan.rpc.auth.LdapManager.ServerType.decode(json['type']) if 'type' in json or not useDefaults else raritan.rpc.auth.LdapManager.ServerType.ACTIVE_DIRECTORY,
                secProto = raritan.rpc.auth.LdapManager.SecurityProtocol.decode(json['secProto']) if 'secProto' in json or not useDefaults else raritan.rpc.auth.LdapManager.SecurityProtocol.SEC_PROTO_NONE,
                port = json['port'] if 'port' in json or not useDefaults else 0,
                sslPort = json['sslPort'] if 'sslPort' in json or not useDefaults else 0,
                forceTrustedCert = json['forceTrustedCert'] if 'forceTrustedCert' in json or not useDefaults else False,
                allowOffTimeRangeCerts = json['allowOffTimeRangeCerts'] if 'allowOffTimeRangeCerts' in json or not useDefaults else False,
                certificate = json['certificate'] if 'certificate' in json or not useDefaults else "",
                adsDomain = json['adsDomain'] if 'adsDomain' in json or not useDefaults else "",
                useAnonymousBind = json['useAnonymousBind'] if 'useAnonymousBind' in json or not useDefaults else False,
                bindDN = json['bindDN'] if 'bindDN' in json or not useDefaults else "",
                bindPwd = json['bindPwd'] if 'bindPwd' in json or not useDefaults else "",
                searchBaseDN = json['searchBaseDN'] if 'searchBaseDN' in json or not useDefaults else "",
                loginNameAttr = json['loginNameAttr'] if 'loginNameAttr' in json or not useDefaults else "",
                userEntryObjClass = json['userEntryObjClass'] if 'userEntryObjClass' in json or not useDefaults else "",
                userSearchFilter = json['userSearchFilter'] if 'userSearchFilter' in json or not useDefaults else "",
                groupInfoInUserEntry = json['groupInfoInUserEntry'] if 'groupInfoInUserEntry' in json or not useDefaults else False,
                supportNestedGroups = json['supportNestedGroups'] if 'supportNestedGroups' in json or not useDefaults else False,
                groupMemberAttr = json['groupMemberAttr'] if 'groupMemberAttr' in json or not useDefaults else "",
                groupEntryObjClass = json['groupEntryObjClass'] if 'groupEntryObjClass' in json or not useDefaults else "",
                groupSearchFilter = json['groupSearchFilter'] if 'groupSearchFilter' in json or not useDefaults else "",
            )
            return obj

        def encode(self):
            json = {}
            json['server'] = str(self.server)
            json['type'] = raritan.rpc.auth.LdapManager.ServerType.encode(self.type)
            json['secProto'] = raritan.rpc.auth.LdapManager.SecurityProtocol.encode(self.secProto)
            json['port'] = self.port
            json['sslPort'] = self.sslPort
            json['forceTrustedCert'] = self.forceTrustedCert
            json['allowOffTimeRangeCerts'] = self.allowOffTimeRangeCerts
            json['certificate'] = str(self.certificate)
            json['adsDomain'] = str(self.adsDomain)
            json['useAnonymousBind'] = self.useAnonymousBind
            json['bindDN'] = str(self.bindDN)
            json['bindPwd'] = str(self.bindPwd)
            json['searchBaseDN'] = str(self.searchBaseDN)
            json['loginNameAttr'] = str(self.loginNameAttr)
            json['userEntryObjClass'] = str(self.userEntryObjClass)
            json['userSearchFilter'] = str(self.userSearchFilter)
            json['groupInfoInUserEntry'] = self.groupInfoInUserEntry
            json['supportNestedGroups'] = self.supportNestedGroups
            json['groupMemberAttr'] = str(self.groupMemberAttr)
            json['groupEntryObjClass'] = str(self.groupEntryObjClass)
            json['groupSearchFilter'] = str(self.groupSearchFilter)
            return json

    class _getLdapServers(Interface.Method):
        name = 'getLdapServers'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = [raritan.rpc.auth.LdapManager.ServerSettings.decode(x0, agent, useDefaults=useDefaults) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.auth.LdapManager.ServerSettings, DecodeException)
            return _ret_

    ERR_CYCLIC_DEP = 1

    ERR_INVALID_CFG = 2

    class _setLdapServers(Interface.Method):
        name = 'setLdapServers'

        @staticmethod
        def encode(serverList):
            for x0 in serverList:
                typecheck.is_struct(x0, raritan.rpc.auth.LdapManager.ServerSettings, AssertionError)
            args = {}
            args['serverList'] = [raritan.rpc.auth.LdapManager.ServerSettings.encode(x0) for x0 in serverList]
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    ERR_SERVER_UNSPECIFIED = 1

    ERR_SERVER_UNREACHABLE = 3

    ERR_AUTHENTICATION_FAILED = 4

    ERR_NO_ROLES = 5

    ERR_NO_KNOWN_ROLES = 6

    class _testLdapServer(Interface.Method):
        name = 'testLdapServer'

        @staticmethod
        def encode(username, password, settings):
            if not typecheck._is_int(username):
                typecheck.is_string(username, AssertionError)
            if not typecheck._is_int(password):
                typecheck.is_string(password, AssertionError)
            typecheck.is_struct(settings, raritan.rpc.auth.LdapManager.ServerSettings, AssertionError)
            args = {}
            args['username'] = str(username)
            args['password'] = str(password)
            args['settings'] = raritan.rpc.auth.LdapManager.ServerSettings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            diagMsg = rsp['diagMsg']
            typecheck.is_int(_ret_, DecodeException)
            if not typecheck._is_int(diagMsg):
                typecheck.is_string(diagMsg, DecodeException)
            return (_ret_, diagMsg)
    def __init__(self, target, agent):
        super(LdapManager, self).__init__(target, agent)
        self.getLdapServers = LdapManager._getLdapServers(self)
        self.setLdapServers = LdapManager._setLdapServers(self)
        self.testLdapServer = LdapManager._testLdapServer(self)

#
# Section generated by IdlC from "RadiusManager.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.auth


# interface
class RadiusManager(Interface):
    idlType = "auth.RadiusManager:4.0.0"

    # enumeration
    class AuthType(Enumeration):
        idlType = "auth.RadiusManager_4_0_0.AuthType:1.0.0"
        values = ["PAP", "CHAP", "MSCHAPv2"]

    AuthType.PAP = AuthType(0)
    AuthType.CHAP = AuthType(1)
    AuthType.MSCHAPv2 = AuthType(2)

    # structure
    class ServerSettings(Structure):
        idlType = "auth.RadiusManager_4_0_0.ServerSettings:1.0.0"
        elements = ["id", "server", "sharedSecret", "udpAuthPort", "udpAccountPort", "timeout", "retries", "authType", "disableAccounting", "messageAuthenticatorOptional"]

        def __init__(self, id="", server="", sharedSecret="", udpAuthPort=0, udpAccountPort=0, timeout=0, retries=0, authType=None, disableAccounting=False, messageAuthenticatorOptional=False):
            if authType is None:
                authType = raritan.rpc.auth.RadiusManager.AuthType.PAP
            if not typecheck._is_int(id):
                typecheck.is_string(id, AssertionError)
            if not typecheck._is_int(server):
                typecheck.is_string(server, AssertionError)
            if not typecheck._is_int(sharedSecret):
                typecheck.is_string(sharedSecret, AssertionError)
            typecheck.is_int(udpAuthPort, AssertionError)
            typecheck.is_int(udpAccountPort, AssertionError)
            typecheck.is_int(timeout, AssertionError)
            typecheck.is_int(retries, AssertionError)
            typecheck.is_enum(authType, raritan.rpc.auth.RadiusManager.AuthType, AssertionError)
            typecheck.is_bool(disableAccounting, AssertionError)
            typecheck.is_bool(messageAuthenticatorOptional, AssertionError)

            self.id = id
            self.server = server
            self.sharedSecret = sharedSecret
            self.udpAuthPort = udpAuthPort
            self.udpAccountPort = udpAccountPort
            self.timeout = timeout
            self.retries = retries
            self.authType = authType
            self.disableAccounting = disableAccounting
            self.messageAuthenticatorOptional = messageAuthenticatorOptional

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                id = json['id'] if 'id' in json or not useDefaults else "",
                server = json['server'] if 'server' in json or not useDefaults else "",
                sharedSecret = json['sharedSecret'] if 'sharedSecret' in json or not useDefaults else "",
                udpAuthPort = json['udpAuthPort'] if 'udpAuthPort' in json or not useDefaults else 0,
                udpAccountPort = json['udpAccountPort'] if 'udpAccountPort' in json or not useDefaults else 0,
                timeout = json['timeout'] if 'timeout' in json or not useDefaults else 0,
                retries = json['retries'] if 'retries' in json or not useDefaults else 0,
                authType = raritan.rpc.auth.RadiusManager.AuthType.decode(json['authType']) if 'authType' in json or not useDefaults else raritan.rpc.auth.RadiusManager.AuthType.PAP,
                disableAccounting = json['disableAccounting'] if 'disableAccounting' in json or not useDefaults else False,
                messageAuthenticatorOptional = json['messageAuthenticatorOptional'] if 'messageAuthenticatorOptional' in json or not useDefaults else False,
            )
            return obj

        def encode(self):
            json = {}
            json['id'] = str(self.id)
            json['server'] = str(self.server)
            json['sharedSecret'] = str(self.sharedSecret)
            json['udpAuthPort'] = self.udpAuthPort
            json['udpAccountPort'] = self.udpAccountPort
            json['timeout'] = self.timeout
            json['retries'] = self.retries
            json['authType'] = raritan.rpc.auth.RadiusManager.AuthType.encode(self.authType)
            json['disableAccounting'] = self.disableAccounting
            json['messageAuthenticatorOptional'] = self.messageAuthenticatorOptional
            return json

    ERR_INVALID_CFG = 1

    ERR_SERVER_UNSPECIFIED = 2

    ERR_INVALID_SHARED_SECRET = 3

    ERR_SERVER_UNREACHABLE = 4

    ERR_AUTHENTICATION_FAILED = 5

    ERR_NO_ROLES = 6

    ERR_NO_KNOWN_ROLES = 7

    ERR_MSG_AUTH_ATTR_MISSING = 8

    class _getRadiusServers(Interface.Method):
        name = 'getRadiusServers'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = [raritan.rpc.auth.RadiusManager.ServerSettings.decode(x0, agent, useDefaults=useDefaults) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.auth.RadiusManager.ServerSettings, DecodeException)
            return _ret_

    class _setRadiusServers(Interface.Method):
        name = 'setRadiusServers'

        @staticmethod
        def encode(serverList):
            for x0 in serverList:
                typecheck.is_struct(x0, raritan.rpc.auth.RadiusManager.ServerSettings, AssertionError)
            args = {}
            args['serverList'] = [raritan.rpc.auth.RadiusManager.ServerSettings.encode(x0) for x0 in serverList]
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _testRadiusServer(Interface.Method):
        name = 'testRadiusServer'

        @staticmethod
        def encode(username, password, settings):
            if not typecheck._is_int(username):
                typecheck.is_string(username, AssertionError)
            if not typecheck._is_int(password):
                typecheck.is_string(password, AssertionError)
            typecheck.is_struct(settings, raritan.rpc.auth.RadiusManager.ServerSettings, AssertionError)
            args = {}
            args['username'] = str(username)
            args['password'] = str(password)
            args['settings'] = raritan.rpc.auth.RadiusManager.ServerSettings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(RadiusManager, self).__init__(target, agent)
        self.getRadiusServers = RadiusManager._getRadiusServers(self)
        self.setRadiusServers = RadiusManager._setRadiusServers(self)
        self.testRadiusServer = RadiusManager._testRadiusServer(self)

#
# Section generated by IdlC from "TacPlusManager.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.auth


# interface
class TacPlusManager(Interface):
    idlType = "auth.TacPlusManager:1.0.1"

    # enumeration
    class AuthenType(Enumeration):
        idlType = "auth.TacPlusManager_1_0_1.AuthenType:1.0.0"
        values = ["ASCII", "PAP", "CHAP", "MSCHAP"]

    AuthenType.ASCII = AuthenType(0)
    AuthenType.PAP = AuthenType(1)
    AuthenType.CHAP = AuthenType(2)
    AuthenType.MSCHAP = AuthenType(3)

    # structure
    class ServerSettings(Structure):
        idlType = "auth.TacPlusManager_1_0_1.ServerSettings:1.0.0"
        elements = ["server", "port", "timeoutSeconds", "retries", "sharedSecret", "authenType", "disableAccounting"]

        def __init__(self, server="", port=0, timeoutSeconds=0, retries=0, sharedSecret="", authenType=None, disableAccounting=False):
            if authenType is None:
                authenType = raritan.rpc.auth.TacPlusManager.AuthenType.ASCII
            if not typecheck._is_int(server):
                typecheck.is_string(server, AssertionError)
            typecheck.is_int(port, AssertionError)
            typecheck.is_int(timeoutSeconds, AssertionError)
            typecheck.is_int(retries, AssertionError)
            if not typecheck._is_int(sharedSecret):
                typecheck.is_string(sharedSecret, AssertionError)
            typecheck.is_enum(authenType, raritan.rpc.auth.TacPlusManager.AuthenType, AssertionError)
            typecheck.is_bool(disableAccounting, AssertionError)

            self.server = server
            self.port = port
            self.timeoutSeconds = timeoutSeconds
            self.retries = retries
            self.sharedSecret = sharedSecret
            self.authenType = authenType
            self.disableAccounting = disableAccounting

        @classmethod
        def decode(cls, json, agent, useDefaults=False):
            obj = cls(
                server = json['server'] if 'server' in json or not useDefaults else "",
                port = json['port'] if 'port' in json or not useDefaults else 0,
                timeoutSeconds = json['timeoutSeconds'] if 'timeoutSeconds' in json or not useDefaults else 0,
                retries = json['retries'] if 'retries' in json or not useDefaults else 0,
                sharedSecret = json['sharedSecret'] if 'sharedSecret' in json or not useDefaults else "",
                authenType = raritan.rpc.auth.TacPlusManager.AuthenType.decode(json['authenType']) if 'authenType' in json or not useDefaults else raritan.rpc.auth.TacPlusManager.AuthenType.ASCII,
                disableAccounting = json['disableAccounting'] if 'disableAccounting' in json or not useDefaults else False,
            )
            return obj

        def encode(self):
            json = {}
            json['server'] = str(self.server)
            json['port'] = self.port
            json['timeoutSeconds'] = self.timeoutSeconds
            json['retries'] = self.retries
            json['sharedSecret'] = str(self.sharedSecret)
            json['authenType'] = raritan.rpc.auth.TacPlusManager.AuthenType.encode(self.authenType)
            json['disableAccounting'] = self.disableAccounting
            return json

    ERR_INVALID_CFG = 1

    ERR_SERVER_UNSPECIFIED = 2

    ERR_INVALID_SHARED_SECRET = 3

    ERR_SERVER_UNREACHABLE = 4

    ERR_AUTHENTICATION_FAILED = 5

    ERR_NO_ROLES = 6

    ERR_NO_KNOWN_ROLES = 7

    class _getTacPlusServers(Interface.Method):
        name = 'getTacPlusServers'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = [raritan.rpc.auth.TacPlusManager.ServerSettings.decode(x0, agent, useDefaults=useDefaults) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.auth.TacPlusManager.ServerSettings, DecodeException)
            return _ret_

    class _setTacPlusServers(Interface.Method):
        name = 'setTacPlusServers'

        @staticmethod
        def encode(serverList):
            for x0 in serverList:
                typecheck.is_struct(x0, raritan.rpc.auth.TacPlusManager.ServerSettings, AssertionError)
            args = {}
            args['serverList'] = [raritan.rpc.auth.TacPlusManager.ServerSettings.encode(x0) for x0 in serverList]
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _testTacPlusServer(Interface.Method):
        name = 'testTacPlusServer'

        @staticmethod
        def encode(username, password, settings):
            if not typecheck._is_int(username):
                typecheck.is_string(username, AssertionError)
            if not typecheck._is_int(password):
                typecheck.is_string(password, AssertionError)
            typecheck.is_struct(settings, raritan.rpc.auth.TacPlusManager.ServerSettings, AssertionError)
            args = {}
            args['username'] = str(username)
            args['password'] = str(password)
            args['settings'] = raritan.rpc.auth.TacPlusManager.ServerSettings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent, useDefaults=False):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(TacPlusManager, self).__init__(target, agent)
        self.getTacPlusServers = TacPlusManager._getTacPlusServers(self)
        self.setTacPlusServers = TacPlusManager._setTacPlusServers(self)
        self.testTacPlusServer = TacPlusManager._testTacPlusServer(self)
